# -*- coding: utf-8 -*-
#
# Copyright (C) 2015 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
VMware VM implementation.
"""

from gns3.qt import QtCore
from gns3.node import Node
from gns3.utils.bring_to_front import bring_window_to_front_from_process_name
from .settings import VMWARE_VM_SETTINGS

import logging
log = logging.getLogger(__name__)


class VMwareVM(Node):

    """
    VirtualBox VM.

    :param module: parent module for this node
    :param server: GNS3 server instance
    :param project: Project instance
    """

    URL_PREFIX = "vmware"
    allocate_vmnet_nio_signal = QtCore.Signal(int, int, str)

    def __init__(self, module, server, project):

        super().__init__(module, server, project)
        self._linked_clone = False

        vmware_vm_settings = {"vmx_path": "",
                              "console": None,
                              "console_host": None,
                              "adapters": VMWARE_VM_SETTINGS["adapters"],
                              "adapter_type": VMWARE_VM_SETTINGS["adapter_type"],
                              "use_any_adapter": VMWARE_VM_SETTINGS["use_any_adapter"],
                              "headless": VMWARE_VM_SETTINGS["headless"],
                              "acpi_shutdown": VMWARE_VM_SETTINGS["acpi_shutdown"],
                              "port_name_format": "Ethernet{0}",
                              "port_segment_size": 0,
                              "first_port_name": None}

        self.settings().update(vmware_vm_settings)

    def _createCallback(self, result):
        """
        Callback for create.

        :param result: server response (dict)
        """
        pass

    def update(self, new_settings):
        """
        Updates the settings for this VMware VM.

        :param new_settings: settings (dict)
        """

        params = {}
        for name, value in new_settings.items():
            if name in self._settings and self._settings[name] != value:
                params[name] = value
        if params:
            self._update(params)

    def info(self):
        """
        Returns information about this VMware VM instance.

        :returns: formatted string
        """

        if self.status() == Node.started:
            state = "started"
        else:
            state = "stopped"

        info = """VMware VM {name} is {state}
  Local node ID is {id}
  Server's node ID is {node_id}
  VMware VM's server runs on {host}, console is on port {console}
""".format(name=self.name(),
           id=self.id(),
           node_id=self._node_id,
           state=state,
           host=self.compute().name(),
           console=self._settings["console"])

        port_info = ""
        for port in self._ports:
            if port.isFree():
                port_info += "     {port_name} is empty\n".format(port_name=port.name())
            else:
                port_info += "     {port_name} {port_description}\n".format(port_name=port.name(),
                                                                            port_description=port.description())
        return info + port_info

    def allocateVMnetInterface(self, port_id):
        """
        Requests an UDP port allocation.

        :param port_id: port identifier
        """

        log.debug("{} is requesting a VMnet interface allocation".format(self.name()))
        self.httpPost("/vmware/nodes/{node_id}/interfaces/vmnet".format(node_id=self._node_id), self._allocateVMnetInterfaceCallback, context={"port_id": port_id})

    def _allocateVMnetInterfaceCallback(self, result, error=False, context={}, **kwargs):
        """
        Callback for allocateVMnetInterface

        :param result: server response (dict)
        :param error: indicates an error (boolean)
        """

        if error:
            log.error("error while allocating a VMnet interface for {}: {}".format(self.name(), result["message"]))
            self.server_error_signal.emit(self.id(), result["message"])
        else:
            port_id = context["port_id"]
            vmnet = result["vmnet"]
            log.debug("{} has allocated VMnet interface {}".format(self.name(), vmnet))
            self.allocate_vmnet_nio_signal.emit(self.id(), port_id, vmnet)

    def console(self):
        """
        Returns the console port for this VMware VM instance.

        :returns: port (integer)
        """

        return self._settings["console"]

    def bringToFront(self):
        """
        Bring the VM window to front.
        """

        if self.status() == Node.started:
            try:
                vmx_pairs = self.module().parseVMwareFile(self.settings()["vmx_path"])
            except OSError as e:
                log.debug("Could not read VMX file: {}".format(e))
                return
            if "displayname" in vmx_pairs:
                window_name = "{} -".format(vmx_pairs["displayname"])
                # try for both VMware Player and Workstation
                bring_window_to_front_from_process_name("vmplayer.exe", title=window_name)
                bring_window_to_front_from_process_name("vmware.exe", title=window_name)

        # bring any console to front
        return Node.bringToFront(self)

    def configPage(self):
        """
        Returns the configuration page widget to be used by the node properties dialog.

        :returns: QWidget object
        """

        from .pages.vmware_vm_configuration_page import VMwareVMConfigurationPage
        return VMwareVMConfigurationPage

    @staticmethod
    def defaultSymbol():
        """
        Returns the default symbol path for this node.

        :returns: symbol path (or resource).
        """

        return ":/symbols/vmware_guest.svg"

    @staticmethod
    def symbolName():

        return "VMware VM"

    @staticmethod
    def categories():
        """
        Returns the node categories the node is part of (used by the device panel).

        :returns: list of node categories
        """

        return [Node.end_devices]

    def __str__(self):

        return "VMware VM"



# -*- coding: utf-8 -*-
#
# Copyright (C) 2015 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
Configuration page for Docker image preferences.
"""

import copy

from gns3.qt import QtCore, QtGui, QtWidgets, qpartial
from gns3.main_window import MainWindow
from gns3.dialogs.configuration_dialog import ConfigurationDialog
from gns3.compute_manager import ComputeManager
from gns3.controller import Controller

from .. import Docker
from ..settings import DOCKER_CONTAINER_SETTINGS
from ..ui.docker_vm_preferences_page_ui import Ui_DockerVMPreferencesPageWidget
from ..pages.docker_vm_configuration_page import DockerVMConfigurationPage
from ..dialogs.docker_vm_wizard import DockerVMWizard


class DockerVMPreferencesPage(QtWidgets.QWidget, Ui_DockerVMPreferencesPageWidget):
    """
    QWidget preference page for Docker image preferences.
    """

    def __init__(self):
        super().__init__()
        self.setupUi(self)

        self._main_window = MainWindow.instance()
        self._docker_containers = {}
        self._items = []

        self.uiNewDockerVMPushButton.clicked.connect(self._dockerImageNewSlot)
        self.uiEditDockerVMPushButton.clicked.connect(self._dockerImageEditSlot)
        self.uiDeleteDockerVMPushButton.clicked.connect(self._dockerImageDeleteSlot)
        self.uiDockerVMsTreeWidget.itemSelectionChanged.connect(self._dockerImageChangedSlot)

    def _createSectionItem(self, name):
        section_item = QtWidgets.QTreeWidgetItem(self.uiDockerVMInfoTreeWidget)
        section_item.setText(0, name)
        font = section_item.font(0)
        font.setBold(True)
        section_item.setFont(0, font)
        return section_item

    def _refreshInfo(self, docker_image):
        self.uiDockerVMInfoTreeWidget.clear()

        # fill out the General section
        section_item = self._createSectionItem("General")
        QtWidgets.QTreeWidgetItem(section_item, ["Image name:", docker_image["image"]])
        try:
            QtWidgets.QTreeWidgetItem(section_item, ["Server:", ComputeManager.instance().getCompute(docker_image["server"]).name()])
        except KeyError:
            pass
        QtWidgets.QTreeWidgetItem(section_item, ["Console type:", str(docker_image["console_type"])])
        QtWidgets.QTreeWidgetItem(section_item, ["Default name format:", docker_image["default_name_format"]])
        QtWidgets.QTreeWidgetItem(section_item, ["Adapters:", str(docker_image["adapters"])])
        if docker_image["start_command"]:
            QtWidgets.QTreeWidgetItem(section_item, ["Start command:", str(docker_image["start_command"])])
        if docker_image["environment"]:
            QtWidgets.QTreeWidgetItem(section_item, ["Environment:", str(docker_image["environment"])])

        if docker_image["extra_hosts"]:
            QtWidgets.QTreeWidgetItem(section_item, ["Extra hosts:", str(docker_image["extra_hosts"])])

        self.uiDockerVMInfoTreeWidget.expandAll()
        self.uiDockerVMInfoTreeWidget.resizeColumnToContents(0)
        self.uiDockerVMInfoTreeWidget.resizeColumnToContents(1)
        self.uiDockerVMsTreeWidget.setMaximumWidth(self.uiDockerVMsTreeWidget.sizeHintForColumn(0) + 10)

    def _dockerImageChangedSlot(self):
        """
        Loads a selected Docker image from the tree widget.
        """

        selection = self.uiDockerVMsTreeWidget.selectedItems()
        self.uiDeleteDockerVMPushButton.setEnabled(len(selection) != 0)
        single_selected = len(selection) == 1
        self.uiEditDockerVMPushButton.setEnabled(single_selected)

        if single_selected:
            key = selection[0].data(0, QtCore.Qt.UserRole)
            docker_image = self._docker_containers[key]
            self._refreshInfo(docker_image)
        else:
            self.uiDockerVMInfoTreeWidget.clear()

    def _dockerImageNewSlot(self):
        """
        Creates a new Docker image.
        """
        wizard = DockerVMWizard(self._docker_containers, parent=self)
        wizard.show()
        if wizard.exec_():
            new_image_settings = wizard.getSettings()
            key = "{server}:{name}".format(server=new_image_settings["server"], name=new_image_settings["name"])
            self._docker_containers[key] = DOCKER_CONTAINER_SETTINGS.copy()
            self._docker_containers[key].update(new_image_settings)

            item = QtWidgets.QTreeWidgetItem(self.uiDockerVMsTreeWidget)
            item.setText(0, self._docker_containers[key]["name"])
            Controller.instance().getSymbolIcon(self._docker_containers[key]["symbol"], qpartial(self._setItemIcon, item))
            item.setData(0, QtCore.Qt.UserRole, key)
            self._items.append(item)
            self.uiDockerVMsTreeWidget.setCurrentItem(item)

    def _dockerImageEditSlot(self):
        """
        Edits a Docker image.
        """

        item = self.uiDockerVMsTreeWidget.currentItem()
        if item:
            key = item.data(0, QtCore.Qt.UserRole)
            docker_image = self._docker_containers[key]
            dialog = ConfigurationDialog(docker_image["name"], docker_image, DockerVMConfigurationPage(), parent=self)
            dialog.show()
            if dialog.exec_():
                # update the icon
                Controller.instance().getSymbolIcon(docker_image["symbol"], qpartial(self._setItemIcon, item))
                if docker_image["name"] != item.text(0):
                    new_key = "{server}:{name}".format(server=docker_image["server"], name=docker_image["name"])
                    if new_key in self._docker_containers:
                        QtWidgets.QMessageBox.critical(self, "Docker image", "Docker container name {} already exists for server {}".format(docker_image["name"],
                                                                                                                                            docker_image["server"]))
                        docker_image["name"] = item.text(0)
                        return
                    self._docker_containers[new_key] = self._docker_containers[key]
                    del self._docker_containers[key]
                    item.setText(0, docker_image["name"])
                    item.setData(0, QtCore.Qt.UserRole, new_key)
                self._refreshInfo(docker_image)

    def _dockerImageDeleteSlot(self):
        """
        Deletes a Docker image.
        """

        for item in self.uiDockerVMsTreeWidget.selectedItems():
            if item:
                key = item.data(0, QtCore.Qt.UserRole)
                del self._docker_containers[key]
                self.uiDockerVMsTreeWidget.takeTopLevelItem(self.uiDockerVMsTreeWidget.indexOfTopLevelItem(item))

    def loadPreferences(self):
        """
        Loads the Docker VM preferences.
        """

        docker_module = Docker.instance()
        self._docker_containers = copy.deepcopy(docker_module.VMs())
        self._items.clear()

        for key, docker_image in self._docker_containers.items():
            item = QtWidgets.QTreeWidgetItem(self.uiDockerVMsTreeWidget)
            item.setText(0, docker_image["name"])

            Controller.instance().getSymbolIcon(docker_image["symbol"], qpartial(self._setItemIcon, item))

            item.setData(0, QtCore.Qt.UserRole, key)
            self._items.append(item)

        if self._items:
            self.uiDockerVMsTreeWidget.setCurrentItem(self._items[0])
            self.uiDockerVMsTreeWidget.sortByColumn(0, QtCore.Qt.AscendingOrder)
            self.uiDockerVMsTreeWidget.setMaximumWidth(self.uiDockerVMsTreeWidget.sizeHintForColumn(0) + 10)

    def savePreferences(self):
        """
        Saves the Docker image preferences.
        """

        Docker.instance().setVMs(self._docker_containers)

    def _setItemIcon(self, item, icon):
        item.setIcon(0, icon)
        self.uiDockerVMsTreeWidget.setMaximumWidth(self.uiDockerVMsTreeWidget.sizeHintForColumn(0) + 10)



# -*- coding: utf-8 -*-
#
# Copyright (C) 2014 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
To normalize a filename and ensure valid characters are used.
(stolen from Django: https://github.com/django/django/blob/master/django/utils/text.py#L439).
"""

import unicodedata
import re


def normalize_filename(filename):
    """
    Normalizes a filename.

    :param filename: filename to normalize
    :return: normalized filename
    """

    filename = unicodedata.normalize('NFKD', filename).encode('ascii', 'ignore').decode('ascii')
    filename = re.sub('[^\w\s-]', '', filename).strip()
    return re.sub('[-\s]+', '-', filename)



# -*- coding: utf-8 -*-
#
# Copyright (C) 2018 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
TraceNG node implementation.
"""

from gns3.node import Node
from gns3.qt import QtWidgets

import logging
log = logging.getLogger(__name__)


class TraceNGNode(Node):
    """
    TraceNG node.

    :param module: parent module for this node
    :param server: GNS3 server instance
    :param project: Project instance
    """
    URL_PREFIX = "traceng"

    def __init__(self, module, server, project):
        super().__init__(module, server, project)

        traceng_settings = {"console_host": None,
                            "console": None,
                            "console_type": "none",
                            "ip_address": "",
                            "default_destination": ""}

        self._last_destination = ""
        self.settings().update(traceng_settings)

    def update(self, new_settings):
        """
        Updates the settings for this TraceNG node.

        :param new_settings: settings dictionary
        """

        params = {}
        for name, value in new_settings.items():
            if name in self._settings and self._settings[name] != value:
                params[name] = value

        if params:
            self._update(params)

    def start(self):
        """
        Starts this node instance.
        """

        if self.isStarted():
            log.debug("{} is already running".format(self.name()))
            return

        if self._last_destination:
            destination = self._last_destination
        else:
            destination = self.settings()["default_destination"]
        destination, ok = QtWidgets.QInputDialog.getText(self.parent(), "TraceNG", "Destination host or IP address:", text=destination)
        if ok:
            if not destination:
                QtWidgets.QMessageBox.critical(self, "TraceNG", "Please provide a host or IP address to trace")
                return
            ip_address = self.settings()["ip_address"]
            if destination == ip_address:
                QtWidgets.QMessageBox.critical(self, "TraceNG", "Destination cannot be the same as this node IP address ({})".format(ip_address))
                return
            self._last_destination = destination
            params = {"destination": destination}
            log.debug("{} is starting".format(self.name()))
            self.controllerHttpPost("/nodes/{node_id}/start".format(node_id=self._node_id), self._startCallback, body=params, timeout=None, progressText="{} is starting".format(self.name()))

    def info(self):
        """
        Returns information about this TraceNG node.

        :returns: formatted string
        """

        if self.status() == Node.started:
            state = "started"
        else:
            state = "stopped"

        info = """Node {name} is {state}
  Local node ID is {id}
  Server's VPCS node ID is {node_id}
  TraceNG's server runs on {host}, console is on port {console}
""".format(name=self.name(),
           id=self.id(),
           node_id=self._node_id,
           state=state,
           host=self.compute().name(),
           console=self._settings["console"])

        port_info = ""
        for port in self._ports:
            if port.isFree():
                port_info += "     {port_name} is empty\n".format(port_name=port.name())
            else:
                port_info += "     {port_name} {port_description}\n".format(port_name=port.name(),
                                                                            port_description=port.description())

        return info + port_info

    def console(self):
        """
        Returns the console port for this TraceNG node.

        :returns: port (integer)
        """

        return self._settings["console"]

    def configPage(self):
        """
        Returns the configuration page widget to be used by the node properties dialog.

        :returns: QWidget object
        """

        from .pages.traceng_node_configuration_page import TraceNGNodeConfigurationPage
        return TraceNGNodeConfigurationPage

    @staticmethod
    def defaultSymbol():
        """
        Returns the default symbol path for this node.

        :returns: symbol path (or resource).
        """

        return ":/symbols/traceng.svg"

    @staticmethod
    def symbolName():

        return "TraceNG"

    @staticmethod
    def categories():
        """
        Returns the node categories the node is part of (used by the node panel).

        :returns: list of node categories
        """

        return [Node.end_devices]

    def __str__(self):

        return "TraceNG node"



#!/usr/bin/env python
#
# Copyright (C) 2016 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import xml.etree.ElementTree as ET

from gns3.items.text_item import TextItem
from gns3.qt import QtGui, QtCore


def test_toSvg(project, controller):
    text = TextItem(project=project)
    text.setPlainText("Hello")
    svg = ET.fromstring(text.toSvg())
    # Travis don't have the font it's broke the CI
    #assert float(svg.get("width")) ==  34.0
    #assert float(svg.get("height")) ==  20.0

    text = svg[0]
    assert text.get("font-family") == "TypeWriter"
    assert text.get("font-size") == "10.0"
    assert text.get("fill") == "#000000"
    assert text.get("fill-opacity") == "1.0"
    assert text.text == "Hello"


def test_fromSvg(project, controller):
    text = TextItem(project=project)
    font = QtGui.QFont()
    font.setPointSizeF(55.8)
    font.setFamily("Verdana")
    font.setBold(True)
    font.setItalic(True)
    font.setStrikeOut(True)
    text.setFont(font)
    text.setDefaultTextColor(QtCore.Qt.red)
    text.setPlainText("Hello")

    text2 = TextItem(project=project)
    text2.fromSvg(text.toSvg())
    assert text2.toPlainText() == "Hello"
    assert hex(text2.defaultTextColor().rgba()) == "0xffff0000"
    assert text2.font().pointSizeF() == 55.8
    assert text2.font().family() == "Verdana"
    assert text2.font().italic()
    assert text2.font().bold()
    assert text2.font().strikeOut()



# -*- coding: utf-8 -*-
#
# Copyright (C) 2014 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
Graphical representation of a note on the QGraphicsScene.
"""

import xml.etree.ElementTree as ET

from ..qt import QtCore, QtWidgets, QtGui
from .drawing_item import DrawingItem
from .utils import colorFromSvg


import logging
log = logging.getLogger(__name__)


class TextItem(QtWidgets.QGraphicsTextItem, DrawingItem):
    """
    Text item for the QGraphicsView.
    """

    def __init__(self, svg=None, **kws):

        super().__init__(**kws)

        from ..main_window import MainWindow

        main_window = MainWindow.instance()
        view_settings = main_window.uiGraphicsView.settings()
        qt_font = QtGui.QFont()
        qt_font.fromString(view_settings["default_label_font"])
        self.setDefaultTextColor(QtGui.QColor(view_settings["default_label_color"]))
        self.setFont(qt_font)

        if svg:
            try:
                svg = self.fromSvg(svg)
            except ET.ParseError as e:
                log.warning(str(e))

        if self._id is None:
            self.create()

    def editText(self):
        """
        Edit mode for this note.
        """

        self.setTextInteractionFlags(QtCore.Qt.TextEditorInteraction)
        self.setSelected(True)
        self.setFocus()
        cursor = self.textCursor()
        cursor.select(QtGui.QTextCursor.Document)
        self.setTextCursor(cursor)

    def mouseDoubleClickEvent(self, event):
        """
        Handles all mouse double click events.

        :param event: QMouseEvent instance
        """

        self.editText()

    def focusOutEvent(self, event):
        """
        Handles all focus out events.

        :param event: QFocusEvent instance
        """

        self.setFlag(QtWidgets.QGraphicsItem.ItemIsFocusable, False)
        cursor = self.textCursor()
        if cursor.hasSelection():
            cursor.clearSelection()
            self.setTextCursor(cursor)
        self.setTextInteractionFlags(QtCore.Qt.NoTextInteraction)
        if not self.toPlainText():
            # delete the note if empty
            self.delete()
            return
        else:
            self.updateDrawing()
        return super().focusOutEvent(event)

    def paint(self, painter, option, widget=None):
        """
        Paints the contents of an item in local coordinates.

        :param painter: QPainter instance
        :param option: QStyleOptionGraphicsItem instance
        :param widget: QWidget instance
        """

        super().paint(painter, option, widget)
        self.drawLayerInfo(painter)

    def setZValue(self, value):
        """
        Sets Z value of the item
        :param value: z layer
        """
        return DrawingItem.setZValue(self, value)

    def toSvg(self):
        """
        Return an SVG version of the text
        """
        svg = ET.Element("svg")
        svg.set("width", str(int(self.boundingRect().width())))
        svg.set("height", str(int(self.boundingRect().height())))

        text = ET.SubElement(svg, "text")
        text.set("font-family", self.font().family())
        text.set("font-size", str(self.font().pointSizeF()))
        if self.font().italic():
            text.set("font-style", "italic")
        if self.font().bold():
            text.set("font-weight", "bold")
        if self.font().strikeOut():
            text.set("text-decoration", "line-through")
        elif self.font().underline():
            text.set("text-decoration", "underline")
        text.set("fill", "#" + hex(self.defaultTextColor().rgba())[4:])
        text.set("fill-opacity", str(self.defaultTextColor().alphaF()))
        text.text = self.toPlainText()

        svg = ET.tostring(svg, encoding="utf-8").decode("utf-8")
        return svg

    def fromSvg(self, svg):

        # sometimes we receive \0 at the end of string inside <svg> element
        try:
            svg = svg.replace("\u0000", "")
        except AttributeError:
            pass

        try:
            svg = ET.fromstring(svg)
        except ET.ParseError:
            self.setPlainText("Unable to parse `text_item`")
            return

        text = svg[0]

        font = QtGui.QFont()
        color = text.get("fill")
        if color:
            new_color = colorFromSvg(color)
            color = self.defaultTextColor()
            color.setBlue(new_color.blue())
            color.setRed(new_color.red())
            color.setGreen(new_color.green())
            self.setDefaultTextColor(color)

        opacity = text.get("fill-opacity")
        if opacity:
            color = self.defaultTextColor()
            color.setAlphaF(float(opacity))
            self.setDefaultTextColor(color)

        font.setPointSizeF(float(text.get("font-size", self.font().pointSizeF())))
        font.setFamily(text.get("font-family", self.font().family()))
        if text.get("font-style") == "italic":
            font.setItalic(True)
        if text.get("font-weight") == "bold":
            font.setBold(True)
        if text.get("text-decoration") == "underline":
            font.setUnderline(True)
        if text.get("text-decoration") == "line-through":
            font.setStrikeOut(True)

        self.setFont(font)
        self.setPlainText(text.text)

    def editable(self):
        """
        Returns either the note is editable or not.

        :return: boolean
        """

        return True

    def keyPressEvent(self, event):
        """
        Handles all key press events

        :param event: QKeyEvent
        """

        if not self.handleKeyPressEvent(event):
            super().keyPressEvent(event)



#!/usr/bin/env python
#
# Copyright (C) 2015 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from unittest.mock import MagicMock, patch
import pytest

from gns3.utils.server_select import server_select


def test_server_select_local_server(main_window, local_server):
    """
    With only local server we don't show the list of server
    """

    with patch("gns3.qt.QtWidgets.QInputDialog.getItem") as mock:
        server = server_select(main_window)

        assert not mock.called
        assert server == local_server


def test_server_select_local_server_local_disallow(main_window, local_server):
    """
    With only local server we don't show the list of server
    """

    with patch("gns3.qt.QtWidgets.QMessageBox.critical"):
        with patch("gns3.qt.QtWidgets.QInputDialog.getItem") as mock:
            server = server_select(main_window, allow_local_server=False)
            assert not mock.called
            assert server is None


def test_server_select_local_server_and_remote_select_local(main_window, remote_server, local_server):

    with patch("gns3.qt.QtWidgets.QInputDialog.getItem", return_value=(local_server.name(), True)) as mock:
        server = server_select(main_window)

        assert mock.called
        args, kwargs = mock.call_args
        assert args[3] == [local_server.name(), remote_server.name()]
        assert server.id() == local_server.id()


def test_server_select_local_server_and_remote_select_remote(main_window, remote_server, local_server):

    with patch("gns3.qt.QtWidgets.QInputDialog.getItem", return_value=(remote_server.name(), True)) as mock:
        server = server_select(main_window)

        assert mock.called
        args, kwargs = mock.call_args
        assert args[3] == [local_server.name(), remote_server.name()]

        assert server == remote_server


def test_server_select_local_server_and_remote_local_disallowed(main_window, remote_server, local_server):

    with patch("gns3.qt.QtWidgets.QInputDialog.getItem", return_value=(remote_server.name(), True)) as mock:
        server = server_select(main_window, allow_local_server=False)

        assert not mock.called
        assert server == remote_server


def test_server_select_local_server_and_remote_user_cancel(main_window, remote_server, local_server):

    with patch("gns3.qt.QtWidgets.QInputDialog.getItem", return_value=(local_server.name(), False)) as mock:
        server = server_select(main_window)

        assert mock.called
        args, kwargs = mock.call_args
        assert server is None



def test_server_select_node_type(main_window, remote_server, local_server):
    """
    If only one server support this node type select it by default
    """

    remote_server.setCapabilities({"node_types": ["dynamips", "nat"]})
    local_server.setCapabilities({"node_types": ["dynamips", "cloud"]})
    with patch("gns3.qt.QtWidgets.QInputDialog.getItem", return_value=(local_server.name(), True)) as mock:
        server = server_select(main_window, node_type="nat")

        assert not mock.called
        assert server == remote_server




# -*- coding: utf-8 -*-
#
# Copyright (C) 2014 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
Configuration page for Docker preferences.
"""

import sys

from gns3.qt import QtWidgets, qpartial
from gns3.controller import Controller

from .. import Docker
from ..ui.docker_preferences_page_ui import Ui_DockerPreferencesPageWidget
from ..settings import DOCKER_SETTINGS


class DockerPreferencesPage(QtWidgets.QWidget, Ui_DockerPreferencesPageWidget):
    """QWidget preference page for Docker."""

    def __init__(self):

        super().__init__()
        self.setupUi(self)

        # connect signals
        self.uiRestoreDefaultsPushButton.clicked.connect(self._restoreDefaultsSlot)

        # if not sys.platform.startswith("linux"):
            # Docker is only supported on Linux

    def _restoreDefaultsSlot(self):
        """Slot to populate the page widgets with the default settings."""
        self._populateWidgets(DOCKER_SETTINGS)

    def _populateWidgets(self, settings):
        """Populates the widgets with the settings.

        :param settings: Docker settings
        """

    def loadPreferences(self):
        """Loads Docker preferences."""
        docker_settings = Docker.instance().settings()
        self._populateWidgets(docker_settings)

    def savePreferences(self):
        """Saves Docker preferences."""
        new_settings = {}
        Docker.instance().setSettings(new_settings)



#!/usr/bin/env python
#
# Copyright (C) 2015 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import sys
import logging

if sys.platform.startswith("win"):
    import psutil
    import pywintypes
    import win32process
    import win32gui
    import win32con

log = logging.getLogger(__name__)


def get_windows_from_pid(pid):

    def callback(hwnd, hwnds):
        if win32gui.IsWindowVisible(hwnd) and win32gui.IsWindowEnabled(hwnd):
            _, found_pid = win32process.GetWindowThreadProcessId(hwnd)
            if found_pid == pid:
                hwnds.append(hwnd)
        return True

    hwnds = []
    win32gui.EnumWindows(callback, hwnds)
    return hwnds


def set_foreground_window(hwnd):

    try:
        if win32gui.IsIconic(hwnd):
            win32gui.ShowWindow(hwnd, win32con.SW_RESTORE)
        if win32gui.IsWindowVisible(hwnd) == 0:
            win32gui.ShowWindow(hwnd, win32con.SW_SHOW)
        win32gui.SetForegroundWindow(hwnd)
    except pywintypes.error as e:
        log.debug("Could not bring window to front '{}'".format(e.strerror))


def bring_window_to_front_from_process_name(process_name, title=None):

    for proc in psutil.process_iter():
        try:
            if proc.name() == process_name:
                for hwnd in get_windows_from_pid(proc.pid):
                    if title is None:
                        set_foreground_window(hwnd)
                        return True
                    elif title in win32gui.GetWindowText(hwnd):
                        set_foreground_window(hwnd)
                        return True
        except psutil.Error:
            continue
    return False


def bring_window_to_front_from_pid(pid):

    for hwnd in get_windows_from_pid(pid):
        set_foreground_window(hwnd)


def bring_window_to_front_from_title(title):

    hwnd = win32gui.FindWindow(None, title)
    if hwnd:
        set_foreground_window(hwnd)
        return True
    return False


def bring_windows_to_front_from_title(title):

    def callback(hwnd, window_title):
        if window_title in str(win32gui.GetWindowText(hwnd)):
            set_foreground_window(hwnd)
    win32gui.EnumWindows(callback, title)



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '/home/grossmj/PycharmProjects/gns3-gui/gns3/modules/vpcs/ui/vpcs_node_configuration_page.ui'
#
# Created: Sun Jul 10 16:55:31 2016
#      by: PyQt5 UI code generator 5.2.1
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_VPCSNodeConfigPageWidget(object):
    def setupUi(self, VPCSNodeConfigPageWidget):
        VPCSNodeConfigPageWidget.setObjectName("VPCSNodeConfigPageWidget")
        VPCSNodeConfigPageWidget.resize(445, 209)
        self.gridLayout = QtWidgets.QGridLayout(VPCSNodeConfigPageWidget)
        self.gridLayout.setObjectName("gridLayout")
        self.uiNameLabel = QtWidgets.QLabel(VPCSNodeConfigPageWidget)
        self.uiNameLabel.setObjectName("uiNameLabel")
        self.gridLayout.addWidget(self.uiNameLabel, 0, 0, 1, 1)
        self.uiNameLineEdit = QtWidgets.QLineEdit(VPCSNodeConfigPageWidget)
        self.uiNameLineEdit.setObjectName("uiNameLineEdit")
        self.gridLayout.addWidget(self.uiNameLineEdit, 0, 2, 1, 1)
        self.uiDefaultNameFormatLabel = QtWidgets.QLabel(VPCSNodeConfigPageWidget)
        self.uiDefaultNameFormatLabel.setObjectName("uiDefaultNameFormatLabel")
        self.gridLayout.addWidget(self.uiDefaultNameFormatLabel, 1, 0, 1, 2)
        self.uiDefaultNameFormatLineEdit = QtWidgets.QLineEdit(VPCSNodeConfigPageWidget)
        self.uiDefaultNameFormatLineEdit.setObjectName("uiDefaultNameFormatLineEdit")
        self.gridLayout.addWidget(self.uiDefaultNameFormatLineEdit, 1, 2, 1, 1)
        self.uiScriptFileLabel = QtWidgets.QLabel(VPCSNodeConfigPageWidget)
        self.uiScriptFileLabel.setObjectName("uiScriptFileLabel")
        self.gridLayout.addWidget(self.uiScriptFileLabel, 2, 0, 1, 2)
        self.horizontalLayout_6 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_6.setObjectName("horizontalLayout_6")
        self.uiScriptFileEdit = QtWidgets.QLineEdit(VPCSNodeConfigPageWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiScriptFileEdit.sizePolicy().hasHeightForWidth())
        self.uiScriptFileEdit.setSizePolicy(sizePolicy)
        self.uiScriptFileEdit.setObjectName("uiScriptFileEdit")
        self.horizontalLayout_6.addWidget(self.uiScriptFileEdit)
        self.uiScriptFileToolButton = QtWidgets.QToolButton(VPCSNodeConfigPageWidget)
        self.uiScriptFileToolButton.setToolButtonStyle(QtCore.Qt.ToolButtonTextOnly)
        self.uiScriptFileToolButton.setObjectName("uiScriptFileToolButton")
        self.horizontalLayout_6.addWidget(self.uiScriptFileToolButton)
        self.gridLayout.addLayout(self.horizontalLayout_6, 2, 2, 1, 1)
        self.uiSymbolLabel = QtWidgets.QLabel(VPCSNodeConfigPageWidget)
        self.uiSymbolLabel.setObjectName("uiSymbolLabel")
        self.gridLayout.addWidget(self.uiSymbolLabel, 3, 0, 1, 2)
        self.horizontalLayout_7 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_7.setObjectName("horizontalLayout_7")
        self.uiSymbolLineEdit = QtWidgets.QLineEdit(VPCSNodeConfigPageWidget)
        self.uiSymbolLineEdit.setObjectName("uiSymbolLineEdit")
        self.horizontalLayout_7.addWidget(self.uiSymbolLineEdit)
        self.uiSymbolToolButton = QtWidgets.QToolButton(VPCSNodeConfigPageWidget)
        self.uiSymbolToolButton.setToolButtonStyle(QtCore.Qt.ToolButtonTextOnly)
        self.uiSymbolToolButton.setObjectName("uiSymbolToolButton")
        self.horizontalLayout_7.addWidget(self.uiSymbolToolButton)
        self.gridLayout.addLayout(self.horizontalLayout_7, 3, 2, 1, 1)
        self.uiCategoryLabel = QtWidgets.QLabel(VPCSNodeConfigPageWidget)
        self.uiCategoryLabel.setObjectName("uiCategoryLabel")
        self.gridLayout.addWidget(self.uiCategoryLabel, 4, 0, 1, 2)
        self.uiCategoryComboBox = QtWidgets.QComboBox(VPCSNodeConfigPageWidget)
        self.uiCategoryComboBox.setObjectName("uiCategoryComboBox")
        self.gridLayout.addWidget(self.uiCategoryComboBox, 4, 2, 1, 1)
        spacerItem = QtWidgets.QSpacerItem(263, 212, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout.addItem(spacerItem, 5, 1, 1, 2)

        self.retranslateUi(VPCSNodeConfigPageWidget)
        QtCore.QMetaObject.connectSlotsByName(VPCSNodeConfigPageWidget)

    def retranslateUi(self, VPCSNodeConfigPageWidget):
        _translate = QtCore.QCoreApplication.translate
        VPCSNodeConfigPageWidget.setWindowTitle(_translate("VPCSNodeConfigPageWidget", "VPCS device configuration"))
        self.uiNameLabel.setText(_translate("VPCSNodeConfigPageWidget", "Name:"))
        self.uiDefaultNameFormatLabel.setText(_translate("VPCSNodeConfigPageWidget", "Default name format:"))
        self.uiScriptFileLabel.setText(_translate("VPCSNodeConfigPageWidget", "Base script file:"))
        self.uiScriptFileToolButton.setText(_translate("VPCSNodeConfigPageWidget", "&Browse..."))
        self.uiSymbolLabel.setText(_translate("VPCSNodeConfigPageWidget", "Symbol:"))
        self.uiSymbolToolButton.setText(_translate("VPCSNodeConfigPageWidget", "&Browse..."))
        self.uiCategoryLabel.setText(_translate("VPCSNodeConfigPageWidget", "Category:"))




# -*- coding: utf-8 -*-
#
# Copyright (C) 2014 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import sys
import psutil
import os
import platform
import struct

try:
    import raven
    from raven.transport.http import HTTPTransport
    RAVEN_AVAILABLE = True
except ImportError:
    # raven is not installed with deb package in order to simplify packaging
    RAVEN_AVAILABLE = False

from .utils.get_resource import get_resource
from .version import __version__, __version_info__

import logging
log = logging.getLogger(__name__)


# Dev build
if __version_info__[3] != 0:
    import faulthandler
    # Display a traceback in case of segfault crash. Usefull when frozen
    # Not enabled by default for security reason
    log.debug("Enable catching segfault")
    faulthandler.enable()


class CrashReport:

    """
    Report crash to a third party service
    """

    DSN = "sync+https://4d0138fdd3e642628228ed99c08a76d6:f51fa98dfa3c45b29de6637d176a3132@sentry.io/38506"
    if hasattr(sys, "frozen"):
        cacert = get_resource("cacert.pem")
        if cacert is not None and os.path.isfile(cacert):
            DSN += "?ca_certs={}".format(cacert)
        else:
            log.warning("The SSL certificate bundle file '{}' could not be found".format(cacert))
    _instance = None

    def __init__(self):
        # We don't want sentry making noise if an error is catched when you don't have internet
        sentry_errors = logging.getLogger('sentry.errors')
        sentry_errors.disabled = True

        sentry_uncaught = logging.getLogger('sentry.errors.uncaught')
        sentry_uncaught.disabled = True

    def captureException(self, exception, value, tb):
        from .local_server import LocalServer
        from .local_config import LocalConfig

        local_server = LocalServer.instance().localServerSettings()
        if local_server["report_errors"]:
            if not RAVEN_AVAILABLE:
                return

            if os.path.exists(LocalConfig.instance().runAsRootPath()):
                log.warning("User has run application as root. Crash reports are disabled.")
                sys.exit(1)
                return

            if os.path.exists(".git"):
                log.warning("A .git directory exist crash report is turn off for developers. Instant exit")
                sys.exit(1)
                return

            if hasattr(exception, "fingerprint"):
                client = raven.Client(CrashReport.DSN, release=__version__, fingerprint=['{{ default }}', exception.fingerprint], transport=HTTPTransport)
            else:
                client = raven.Client(CrashReport.DSN, release=__version__, transport=HTTPTransport)
            context = {
                "os:name": platform.system(),
                "os:release": platform.release(),
                "os:win_32": " ".join(platform.win32_ver()),
                "os:mac": "{} {}".format(platform.mac_ver()[0], platform.mac_ver()[2]),
                "os:linux": " ".join(platform.linux_distribution()),
                "python:version": "{}.{}.{}".format(sys.version_info[0],
                                                    sys.version_info[1],
                                                    sys.version_info[2]),
                "python:bit": struct.calcsize("P") * 8,
                "python:encoding": sys.getdefaultencoding(),
                "python:frozen": "{}".format(hasattr(sys, "frozen"))
            }
            context = self._add_qt_information(context)
            client.tags_context(context)
            try:
                report = client.captureException((exception, value, tb))
            except Exception as e:
                log.error("Can't send crash report to Sentry: {}".format(e))
                return
            log.debug("Crash report sent with event ID: {}".format(client.get_ident(report)))

    def _add_qt_information(self, context):
        try:
            from .qt import QtCore
            import sip
        except ImportError:
            return context
        context["psutil:version"] = psutil.__version__
        context["pyqt:version"] = QtCore.PYQT_VERSION_STR
        context["qt:version"] = QtCore.QT_VERSION_STR
        context["sip:version"] = sip.SIP_VERSION_STR
        return context

    @classmethod
    def instance(cls):
        if cls._instance is None:
            cls._instance = CrashReport()
        return cls._instance



# -*- coding: utf-8 -*-
#
# Copyright (C) 2015 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
import hashlib

import logging
log = logging.getLogger(__name__)


class Image:
    """
    An appliance image file.
    """

    # Cache md5sum in order to improve performances
    _cache = {}

    def __init__(self, emulator, path, filename=None):
        """
        :params: Emulator type
        :params: path of the image
        """

        self._location = "local"
        self._emulator = emulator
        self.path = path
        if filename is None:
            self._filename = os.path.basename(self.path)
        else:
            self._filename = filename
        self._md5sum = None
        self._version = None
        self._filesize = None

    @property
    def location(self):
        """
        :returns: remote or local. Where the file is store.
        If local we will need to upload it at the end of the process
        """
        return self._location

    @location.setter
    def location(self, val):
        self._location = val

    @property
    def filename(self):
        """
        :returns: Image filename
        """
        return self._filename

    @property
    def version(self):
        """
        :returns: Get the file version / release
        """
        return self._version

    @version.setter
    def version(self, version):
        """
        :returns: Set the file version / release
        """
        self._version = version

    @property
    def md5sum(self):
        """
        Compute a md5 hash for file

        :params cache: Cache sum on disk
        :returns: hexadecimal md5
        """

        if self._md5sum is None:
            from_cache = Image._cache.get(self.path)
            if from_cache:
                self._md5sum = from_cache
                return self._md5sum

            if os.path.exists(self.path + ".md5sum"):
                with open(self.path + ".md5sum", encoding="utf-8") as f:
                    self._md5sum = f.read().strip()
                    return self._md5sum

            if not os.path.isfile(self.path):
                return None
            m = hashlib.md5()
            with open(self.path, "rb") as f:
                while True:
                    buf = f.read(4096)
                    if not buf:
                        break
                    m.update(buf)
            self._md5sum = m.hexdigest()
        Image._cache[self.path] = self._md5sum
        return self._md5sum

    @md5sum.setter
    def md5sum(self, val):
        self._md5sum = val

    @property
    def filesize(self):
        """
        Return image file size
        """
        if self._filesize is not None:
            return self._filesize
        try:
            self._filesize = os.path.getsize(self.path)
            return self._filesize
        except OSError:
            return 0

    @filesize.setter
    def filesize(self, val):
        self._filesize = val

    @property
    def emulator(self):
        return self._emulator



# -*- coding: utf-8 -*-
#
# Copyright (C) 2014 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
Base class for Dynamips router implementation on the client side.
"""

import os
import re

from gns3.node import Node
from gns3.utils.normalize_filename import normalize_filename

from ..adapters import ADAPTER_MATRIX
from ..wics import WIC_MATRIX

import logging
log = logging.getLogger(__name__)


class Router(Node):

    """
    Dynamips router (client implementation).

    :param module: parent module for this node
    :param server: GNS3 server instance
    :param project: Project instance
    :param platform: c7200, c3745, c3725, c3600, c2691, c2600 or c1700
    """

    URL_PREFIX = "dynamips"

    def __init__(self, module, server, project, platform="c7200"):

        super().__init__(module, server, project)
        self._dynamips_id = None

        router_settings = {"platform": platform,
                           "chassis": "",
                           "image": "",
                           "image_md5sum": "",
                           "startup_config": "",
                           "private_config": "",
                           "ram": 128,
                           "nvram": 128,
                           "mmap": True,
                           "sparsemem": True,
                           "clock_divisor": 8,
                           "idlepc": "",
                           "idlemax": 500,
                           "idlesleep": 30,
                           "iomem": 15,
                           "exec_area": 64,
                           "disk0": 0,
                           "disk1": 0,
                           "auto_delete_disks": False,
                           "console": None,
                           "console_host": None,
                           "aux": None,
                           "mac_addr": None,
                           "system_id": "FTX0945W0MY",
                           "slot0": None,
                           "slot1": None,
                           "slot2": None,
                           "slot3": None,
                           "slot4": None,
                           "slot5": None,
                           "slot6": None,
                           "wic0": None,
                           "wic1": None,
                           "wic2": None}

        self.settings().update(router_settings)

    def _createCallback(self, result):
        """
        Callback for create.

        :param result: server response
        """

        self._dynamips_id = result.get("dynamips_id")

    def update(self, new_settings):
        """
        Updates the settings for this router.

        :param new_settings: settings dictionary
        """

        params = {}

        for name, value in new_settings.items():
            if name in self._settings:
                if self._settings[name] != value:
                    params[name] = value
            else:
                # All key should be known
                raise ValueError(name)

        if params:
            self._update(params)

    def _updateCallback(self, result):
        """
        Callback for update.

        :param result: server response
        """

        for name, value in result.items():
            if name in self._settings:
                if self._settings[name] != value:
                    log.debug("{}: updating {} from '{}' to '{}'".format(self.name(), name, self._settings[name], value))
                    self._settings[name] = value
            elif name not in ("project_id", "port_name_format", "port_segment_size", "first_port_name", "node_directory", "status", "node_id", "width", "height", "compute_id", "node_type", "dynamips_id", "command_line"):
                # All key should be known, but we raise error only in debug
                if logging.getLogger().isEnabledFor(logging.DEBUG):
                    raise ValueError(name)

    def computeIdlepcs(self, callback):
        """
        Get idle-PC proposals.
        """

        log.debug("{} is requesting Idle-PC proposals".format(self.name()))
        self.controllerHttpGet("/nodes/{node_id}/dynamips/idlepc_proposals".format(
            node_id=self._node_id),
            callback,
            timeout=240,
            context={"router": self},
            progressText="Computing Idle-PC values, please wait...")

    def computeAutoIdlepc(self, callback):
        """
        Find the best idle-PC value.
        """

        log.debug("{} is requesting Idle-PC proposals".format(self.name()))
        self.controllerHttpGet("/nodes/{node_id}/dynamips/auto_idlepc".format(
            node_id=self._node_id),
            callback,
            timeout=240,
            context={"router": self},
            progressText="Computing Idle-PC values, please wait...")

    def idlepc(self):
        """
        Returns the current Idle-PC value for this router.

        :returns: idlepc value (string)
        """

        return self._settings["idlepc"]

    def setIdlepc(self, idlepc):
        """
        Sets a new Idle-PC value for this router.

        :param idlepc: idlepc value (string)
        """

        self.update({"idlepc": idlepc})

    def _slot_info(self):
        """
        Returns information about the slots/ports of this router.

        :returns: formated string
        """

        slots = {}
        slot_info = ""
        for name, value in self._settings.items():
            if name.startswith("slot") and value is not None:
                slots[name] = value

        for name in sorted(slots.keys()):
            slot_number = int(name[-1])
            adapter_name = slots[name]
            nb_ports = ADAPTER_MATRIX[adapter_name]["nb_ports"]
            if nb_ports == 1:
                port_string = "port"
            else:
                port_string = "ports"

            slot_info = slot_info + "   slot {slot_number} hardware is {adapter} with {nb_ports} {port_string}\n".format(slot_number=str(slot_number),
                                                                                                                         adapter=adapter_name,
                                                                                                                         nb_ports=nb_ports,
                                                                                                                         port_string=port_string)

            port_names = {}
            for port in self._ports:
                if port.adapterNumber() == slot_number and port.portNumber() < 16:
                    port_names[port.name()] = port
            sorted_ports = sorted(port_names.keys())

            for port_name in sorted_ports:
                port_info = port_names[port_name]
                if port_info.isFree():
                    slot_info += "     {} is empty\n".format(port_name)
                else:
                    slot_info += "     {port_name} {port_description}\n".format(port_name=port_name,
                                                                                port_description=port_info.description())

        wic_slots = {}
        for name, value in self._settings.items():
            if name.startswith("wic") and value is not None:
                wic_slots[name] = value

        for name in sorted(wic_slots.keys()):
            wic_slot_number = int(name[-1])
            wic_name = wic_slots[name]
            nb_ports = WIC_MATRIX[wic_name]["nb_ports"]
            if nb_ports == 1:
                port_string = "port"
            else:
                port_string = "ports"

            slot_info = slot_info + "   {wic_name} installed in WIC slot {wic_slot_number} with {nb_ports} {port_string}\n".format(wic_name=wic_name,
                                                                                                                                   wic_slot_number=wic_slot_number,
                                                                                                                                   nb_ports=nb_ports,
                                                                                                                                   port_string=port_string)

            base = 16 * (wic_slot_number + 1)
            port_names = {}
            for port_number in range(0, nb_ports):
                for port in self._ports:
                    if port.adapterNumber() == 0 and port.portNumber() == base + port_number:
                        port_names[port.name()] = port
            sorted_ports = sorted(port_names.keys())

            for port_name in sorted_ports:
                port_info = port_names[port_name]
                if port_info.isFree():
                    slot_info += "     {} is empty\n".format(port_name)
                else:
                    slot_info += "     {port_name} {port_description}\n".format(port_name=port_name,
                                                                                port_description=port_info.description())

        return slot_info

    def info(self):
        """
        Returns information about this router.

        :returns: formated string
        """

        if self.status() == Node.started:
            state = "started"
        elif self.status() == Node.suspended:
            state = "suspended"
        else:
            state = "stopped"

        platform = self._settings["platform"]
        router_specific_info = ""
        if platform == "c7200":
            router_specific_info = "{midplane} {npe}".format(midplane=self._settings["midplane"],
                                                             npe=self._settings["npe"])

            router_specific_info = router_specific_info.upper()

        # get info about Idle-PC
        idlepc_info = "with no idlepc value"
        if self._settings["idlepc"]:
            idlepc_info = "with idlepc value of {idlepc}, idlemax of {idlemax} and idlesleep of {idlesleep} ms".format(idlepc=self._settings["idlepc"],
                                                                                                                       idlemax=self._settings["idlemax"],
                                                                                                                       idlesleep=self._settings["idlesleep"])

        info = """Router {name} is {state}
  Local node ID is {id}
  Server's node ID is {node_id}
  Dynamips ID is {dynamips_id}
  Hardware is Dynamips emulated Cisco {platform} {specific_info} with {ram} MB RAM and {nvram} KB NVRAM
  Router's server runs on {host}, console is on port {console}, aux is on port {aux}
  Image is {image_name}
  {idlepc_info}
  {disk0} MB disk0 size, {disk1} MB disk1 size
""".format(name=self.name(),
           id=self.id(),
           node_id=self._node_id,
           dynamips_id=self._dynamips_id,
           state=state,
           platform=platform,
           specific_info=router_specific_info,
           ram=self._settings["ram"],
           nvram=self._settings["nvram"],
           host=self.compute().name(),
           console=self._settings["console"],
           aux=self._settings["aux"],
           image_name=os.path.basename(self._settings["image"]),
           idlepc_info=idlepc_info,
           disk0=self._settings["disk0"],
           disk1=self._settings["disk1"])

        # gather information about PA, their interfaces and connections
        slot_info = self._slot_info()
        return info + slot_info

    def configFiles(self):
        """
        Name of the configuration files
        """
        return [
            "configs/i{}_startup-config.cfg".format(self._dynamips_id),
            "configs/i{}_private-config.cfg".format(self._dynamips_id)
        ]

    def console(self):
        """
        Returns the console port for this router.

        :returns: port (integer)
        """

        return self._settings["console"]

    def auxConsole(self):
        """
        Returns the auxiliary console port for this router.

        :returns: port (integer)
        """

        return self._settings["aux"]

    def configPage(self):
        """
        Returns the configuration page widget to be used by the node properties dialog.

        :returns: QWidget object
        """

        from ..pages.ios_router_configuration_page import IOSRouterConfigurationPage
        return IOSRouterConfigurationPage

    @staticmethod
    def validateHostname(hostname):
        """
        Checks if the hostname is valid.

        :param hostname: hostname to check

        :returns: boolean
        """

        # IOS names must start with a letter, end with a letter or digit, and
        # have as interior characters only letters, digits, and hyphens.
        # They must be 63 characters or fewer.
        if re.search(r"""^[\-\w]+$""", hostname) and len(hostname) <= 63:
            return True
        return False

    @staticmethod
    def defaultSymbol():
        """
        Returns the default symbol path for this router.

        :returns: symbol path (or resource).
        """

        return ":/symbols/router.svg"

    @staticmethod
    def categories():
        """
        Returns the node categories the node is part of (used by the device panel).

        :returns: list of node categories
        """

        return [Node.routers]



# -*- coding: utf-8 -*-
#
# Copyright (C) 2015 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Wizard for Docker images."""

import sys

from gns3.qt import QtGui, QtWidgets
from gns3.dialogs.vm_wizard import VMWizard
from gns3.compute_manager import ComputeManager

from ..ui.docker_vm_wizard_ui import Ui_DockerVMWizard
from .. import Docker


class DockerVMWizard(VMWizard, Ui_DockerVMWizard):
    """Wizard to create a Docker image.

    :param docker_containers: existing Docker images
    :param parent: parent widget
    """

    def __init__(self, docker_containers, parent):

        super().__init__(docker_containers, parent)
        self._docker_containers = docker_containers
        self.setPixmap(QtWidgets.QWizard.LogoPixmap, QtGui.QPixmap(":/icons/docker.png"))

        self.uiNewImageRadioButton.setChecked(True)
        self._existingImageRadioButtonToggledSlot(False)
        self.uiExistingImageRadioButton.toggled.connect(self._existingImageRadioButtonToggledSlot)

        if ComputeManager.instance().localPlatform().startswith("win") or ComputeManager.instance().localPlatform().startswith("darwin"):
            # Cannot use Docker locally on Windows and Mac
            self._disableLocalServer()

    def _existingImageRadioButtonToggledSlot(self, status):
        if self.uiExistingImageRadioButton.isChecked():
            self.uiImageLineEdit.hide()
            self.uiImageNameLabel.hide()
            self.uiImageListLabel.show()
            self.uiImageListComboBox.show()
        else:
            self.uiImageNameLabel.show()
            self.uiImageLineEdit.show()
            self.uiImageListLabel.hide()
            self.uiImageListComboBox.hide()

    def initializePage(self, page_id):

        super().initializePage(page_id)

        if self.page(page_id) == self.uiImageWizardPage:
            Docker.instance().getDockerImagesFromServer(self._compute_id, self._getDockerImagesFromServerCallback)

    def _getDockerImagesFromServerCallback(
            self, result, error=False, **kwargs):
        """Callback for getDockerImagesFromServer.

        :param progress_dialog: QProgressDialog instance
        :param result: server response
        :param error: indicates an error (boolean)
        """
        if error:
            QtWidgets.QMessageBox.critical(self, "Docker Images", "{}".format(result["message"]))
        else:
            self.uiImageListComboBox.clear()
            if len(result) == 0:
                self.uiNewImageRadioButton.setChecked(True)
            else:
                self.uiExistingImageRadioButton.setChecked(True)
                for image in result:
                    self.uiImageListComboBox.addItem(image["image"], image)

    def validateCurrentPage(self):
        """Validates the server."""

        if super().validateCurrentPage() is False:
            return False

        if self.currentPage() == self.uiImageWizardPage:
            if self.uiImageListComboBox.currentIndex() < 0 and self.uiExistingImageRadioButton.isChecked():
                QtWidgets.QMessageBox.critical(
                    self, "Docker images",
                    "There are no Docker images selected!")
                return False
            self.uiNameLineEdit.setText(self._getImageName().split(":")[0].replace("/", "-"))

        if self.currentPage() == self.uiNameWizardPage:
            if "/" in self.uiNameLineEdit.text():
                QtWidgets.QMessageBox.critical(self, "Container name", "/ is not allowed in container name")
                return False
            if self.uiNameLineEdit.text() in [d["name"] for d in self._docker_containers.values()]:
                QtWidgets.QMessageBox.critical(self, "Container name", "This name already exist!")
                return False
        return True

    def _getImageName(self):
        if self.uiExistingImageRadioButton.isChecked():
            index = self.uiImageListComboBox.currentIndex()
            return self.uiImageListComboBox.itemText(index)
        else:
            name = self.uiImageLineEdit.text()
            return name

    def getSettings(self):
        """Returns the settings set in this Wizard.

        :return: settings
        :rtype: dict
        """

        image = self._getImageName()
        start_command = self.uiStartCommandLineEdit.text()
        name = self.uiNameLineEdit.text()

        settings = {
            "image": image,
            "server": self._compute_id,
            "adapters": self.uiAdaptersSpinBox.value(),
            "name": name,
            "environment": self.uiEnvironmentTextEdit.toPlainText(),
            "start_command": start_command,
            "console_type": self.uiConsoleTypeComboBox.currentText(),
        }
        return settings



# -*- coding: utf-8 -*-
#
# Copyright (C) 2014 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from ..qt import QtWidgets, QtCore, qslot, qpartial
from ..topology import Topology
from ..ui.edit_project_dialog_ui import Ui_EditProjectDialog


class EditProjectDialog(QtWidgets.QDialog, Ui_EditProjectDialog):
    """
    Edit current project settings
    """

    def __init__(self, parent):

        super().__init__(parent)
        self.setupUi(self)
        self._project = Topology.instance().project()
        self.uiProjectNameLineEdit.setText(self._project.name())
        self.uiProjectAutoOpenCheckBox.setChecked(self._project.autoOpen())
        self.uiProjectAutoCloseCheckBox.setChecked(not self._project.autoClose())
        self.uiProjectAutoStartCheckBox.setChecked(self._project.autoStart())
        self.uiSceneWidthSpinBox.setValue(self._project.sceneWidth())
        self.uiSceneHeightSpinBox.setValue(self._project.sceneHeight())
        self.uiGridSizeSpinBox.setValue(self._project.gridSize())

        self.uiGlobalVariablesGrid.setAlignment(QtCore.Qt.AlignTop)

        self.uiNewVarButton = QtWidgets.QPushButton('Add new variable', self)
        self.uiNewVarButton.setSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        self.uiNewVarButton.clicked.connect(self.onAddNewVariable)
        self.uiGlobalVariablesGrid.addWidget(self.uiNewVarButton, 0, 3, QtCore.Qt.AlignRight)

        self._variables = self.setUpVariables()
        self.updateGlobalVariables()

    def setUpVariables(self):
        new_variable = {"name": "", "value": ""}
        variables = self._project.variables()

        if variables is not None:
            variables.append(new_variable)
        else:
            variables = [new_variable]
        return variables

    def updateGlobalVariables(self):
        while True:
            item = self.uiGlobalVariablesGrid.takeAt(1)
            if item is None:
                break
            elif item.widget():
                item.widget().deleteLater()

        for i, variable in enumerate(self._variables, start=1):
            nameLabel = QtWidgets.QLabel()
            nameLabel.setText("Name:")
            self.uiGlobalVariablesGrid.addWidget(nameLabel, i, 0)

            nameEdit = QtWidgets.QLineEdit()
            nameEdit.setText(variable.get("name", ""))
            nameEdit.textChanged.connect(qpartial(self.onNameChange, variable))
            self.uiGlobalVariablesGrid.addWidget(nameEdit, i, 1)

            valueLabel = QtWidgets.QLabel()
            valueLabel.setText("Value:")
            self.uiGlobalVariablesGrid.addWidget(valueLabel, i, 2)

            valueEdit = QtWidgets.QLineEdit()
            valueEdit.setText(variable.get("value", ""))
            valueEdit.textChanged.connect(qpartial(self.onValueChange, variable))
            self.uiGlobalVariablesGrid.addWidget(valueEdit, i, 3)

    @qslot
    def onAddNewVariable(self, event):
        self._variables += [{"name": "", "value": ""}]
        self.updateGlobalVariables()

    def onNameChange(self, variable, text):
        variable["name"] = text

    def onValueChange(self, variable, text):
        variable["value"] = text

    def _cleanVariables(self):
        return [v for v in self._variables if v.get("name", "").strip() != ""]

    def done(self, result):
        """
        Called when the dialog is closed.

        :param result: boolean (accepted or rejected)
        """

        if result:
            self._project.setName(self.uiProjectNameLineEdit.text())
            self._project.setAutoOpen(self.uiProjectAutoOpenCheckBox.isChecked())
            self._project.setAutoClose(not self.uiProjectAutoCloseCheckBox.isChecked())
            self._project.setAutoStart(self.uiProjectAutoStartCheckBox.isChecked())
            self._project.setSceneHeight(self.uiSceneHeightSpinBox.value())
            self._project.setSceneWidth(self.uiSceneWidthSpinBox.value())
            self._project.setGridSize(self.uiGridSizeSpinBox.value())
            self._project.setVariables(self._cleanVariables())
            self._project.update()
        super().done(result)



# -*- coding: utf-8 -*-
#
# Copyright (C) 2018 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import copy

from gns3.qt import QtWidgets, QtCore, qpartial
from gns3.ui.project_welcome_dialog_ui import Ui_ProjectWelcomeDialog

import logging
log = logging.getLogger(__name__)


class ProjectWelcomeDialog(QtWidgets.QDialog, Ui_ProjectWelcomeDialog):
    """
    This dialog shows when project is imported and global variables assigned to the project are missing.
    """

    def __init__(self, parent, project):

        super().__init__(parent)
        self._project = project
        self.setupUi(self)
        self.uiOkButton.clicked.connect(self._okButtonClickedSlot)
        self.gridLayout.setAlignment(QtCore.Qt.AlignTop)
        self.label.setOpenExternalLinks(True)

        self._variables = self._getVariables(project)

        self._loadReadme()
        self._addMisingVariablesEdits()

    def _getVariables(self, project):
        variables = copy.copy(self._project.variables())
        if variables is None:
            variables = []
        return variables

    def _addMisingVariablesEdits(self):
        missing = [v for v in self._variables if v.get("value", "").strip() == ""]
        for i, variable in enumerate(missing, start=0):
            nameLabel = QtWidgets.QLabel()
            nameLabel.setText(variable.get("name", ""))
            self.gridLayout.addWidget(nameLabel, i, 0)

            valueEdit = QtWidgets.QLineEdit()
            valueEdit.setText(variable.get("value", ""))
            valueEdit.textChanged.connect(qpartial(self.onValueChange, variable))
            self.gridLayout.addWidget(valueEdit, i, 1)

    def _loadReadme(self):
        self._project.get("/files/README.txt", self._loadedReadme)

    def _loadedReadme(self, result, error=False, raw_body=None, context={}, **kwargs):
        if not error:
            self.label.setText(raw_body.decode("utf-8"))

    def onValueChange(self, variable, text):
        variable["value"] = text

    def _okButtonClickedSlot(self):
        missing = [v for v in self._variables if v.get("value", "").strip() == ""]
        if len(missing) > 0:
            reply = QtWidgets.QMessageBox.warning(
                self, 'Missing values',
                'Are you sure you want to continue without providing missing values?',
                QtWidgets.QMessageBox.Yes,
                QtWidgets.QMessageBox.No)
            if reply == QtWidgets.QMessageBox.No:
                return

        self._project.setVariables(self._variables)
        self._project.update()
        self.accept()




# -*- coding: utf-8 -*-
#
# Copyright (C) 2014 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
Configuration page for general preferences.
"""

import os
import shutil
import json

from gns3.qt import QtGui, QtCore, QtWidgets
from gns3.local_config import LocalConfig
from ..ui.general_preferences_page_ui import Ui_GeneralPreferencesPageWidget
from gns3.local_server import LocalServer
from ..settings import GRAPHICS_VIEW_SETTINGS, GENERAL_SETTINGS, STYLES
from ..dialogs.console_command_dialog import ConsoleCommandDialog


class GeneralPreferencesPage(QtWidgets.QWidget, Ui_GeneralPreferencesPageWidget):

    """
    QWidget configuration page for general preferences.
    """

    def __init__(self, parent=None):

        super().__init__()
        self.setupUi(self)
        self._remote_servers = {}
        self._preferences_dialog = parent

        # Display the path of the config file
        config_file_path = LocalConfig.instance().configFilePath()
        self.uiConfigurationFileLabel.setText(config_file_path)

        self.uiProjectsPathToolButton.clicked.connect(self._projectsPathSlot)
        self.uiSymbolsPathToolButton.clicked.connect(self._symbolsPathSlot)
        self.uiImagesPathToolButton.clicked.connect(self._imagesPathSlot)
        self.uiConfigsPathToolButton.clicked.connect(self._configsPathSlot)
        self.uiAppliancesPathToolButton.clicked.connect(self._appliancesPathSlot)
        self.uiImportConfigurationFilePushButton.clicked.connect(self._importConfigurationFileSlot)
        self.uiExportConfigurationFilePushButton.clicked.connect(self._exportConfigurationFileSlot)
        self.uiRestoreDefaultsPushButton.clicked.connect(self._restoreDefaultsSlot)
        self.uiTelnetConsolePreconfiguredCommandPushButton.clicked.connect(self._telnetConsolePreconfiguredCommandSlot)
        self.uiVNCConsolePreconfiguredCommandPushButton.clicked.connect(self._vncConsolePreconfiguredCommandSlot)
        self.uiSPICEConsolePreconfiguredCommandPushButton.clicked.connect(self._spiceConsolePreconfiguredCommandSlot)
        self.uiDefaultLabelFontPushButton.clicked.connect(self._setDefaultLabelFontSlot)
        self.uiDefaultLabelColorPushButton.clicked.connect(self._setDefaultLabelColorSlot)
        self.uiBrowseConfigurationPushButton.clicked.connect(self._browseConfigurationDirectorySlot)
        self._default_label_color = QtGui.QColor(QtCore.Qt.black)
        self.uiStyleComboBox.addItems(STYLES)
        self.uiImageDirectoriesAddPushButton.clicked.connect(self._imageDirectoriesAddPushButtonSlot)
        self.uiImageDirectoriesDeletePushButton.clicked.connect(self._imageDirectoriesDeletePushButtonSlot)

    def _imageDirectoriesAddPushButtonSlot(self):
        path = QtWidgets.QFileDialog.getExistingDirectory(self, "My images directory", options=QtWidgets.QFileDialog.ShowDirsOnly)
        if path:
            if self.uiImageDirectoriesListWidget.findItems(path, QtCore.Qt.MatchFixedString):
                QtWidgets.QMessageBox.critical(self, "Images directory", "This directory has already been added")
                return
            count = 0
            for _, _, files in os.walk(path):
                count += len(files)
                if count > 10000:
                    QtWidgets.QMessageBox.warning(self, "Images directory", "This directory contains a lot of files, the scan process could consume a lot of resources")
                    break
            self.uiImageDirectoriesListWidget.addItem(path)

    def _imageDirectoriesDeletePushButtonSlot(self):
        item = self.uiImageDirectoriesListWidget.currentItem()
        if item:
            self.uiImageDirectoriesListWidget.takeItem(self.uiImageDirectoriesListWidget.currentRow())

    def _projectsPathSlot(self):
        """
        Slot to select the projects directory path.
        """

        local_server = LocalServer.instance().localServerSettings()
        directory = local_server["projects_path"]
        path = QtWidgets.QFileDialog.getExistingDirectory(self, "My projects directory", directory, QtWidgets.QFileDialog.ShowDirsOnly)
        if path:
            self.uiProjectsPathLineEdit.setText(path)
            self.uiProjectsPathLineEdit.setCursorPosition(0)

    def _symbolsPathSlot(self):
        """
        Slot to select the symbols directory path.
        """

        local_server = LocalServer.instance().localServerSettings()
        directory = local_server["symbols_path"]
        path = QtWidgets.QFileDialog.getExistingDirectory(self, "My symbols directory", directory, QtWidgets.QFileDialog.ShowDirsOnly)
        if path:
            self.uiSymbolsPathLineEdit.setText(path)
            self.uiSymbolsPathLineEdit.setCursorPosition(0)

    def _imagesPathSlot(self):
        """
        Slot to select the images directory path.
        """

        local_server = LocalServer.instance().localServerSettings()
        directory = local_server["images_path"]
        path = QtWidgets.QFileDialog.getExistingDirectory(self, "My images directory", directory, QtWidgets.QFileDialog.ShowDirsOnly)
        if path:
            self.uiImagesPathLineEdit.setText(path)
            self.uiImagesPathLineEdit.setCursorPosition(0)

    def _configsPathSlot(self):
        """
        Slot to select the configs directory path.
        """

        local_server = LocalServer.instance().localServerSettings()
        directory = local_server["configs_path"]
        path = QtWidgets.QFileDialog.getExistingDirectory(self, "My configs directory", directory, QtWidgets.QFileDialog.ShowDirsOnly)
        if path:
            self.uiConfigsPathLineEdit.setText(path)
            self.uiConfigsPathLineEdit.setCursorPosition(0)

    def _appliancesPathSlot(self):
        """
        Slot to select the appliances directory path.
        """

        local_server = LocalServer.instance().localServerSettings()
        directory = local_server["appliances_path"]
        path = QtWidgets.QFileDialog.getExistingDirectory(self, "My custom appliances directory", directory, QtWidgets.QFileDialog.ShowDirsOnly)
        if path:
            self.uiAppliancesPathLineEdit.setText(path)
            self.uiAppliancesPathLineEdit.setCursorPosition(0)

    def _restoreDefaultsSlot(self):
        """
        Slot to restore default settings
        """

        self._populateGeneralSettingWidgets(GENERAL_SETTINGS)
        self._populateGraphicsViewSettingWidgets(GRAPHICS_VIEW_SETTINGS)

    def _telnetConsolePreconfiguredCommandSlot(self):
        """
        Slot to set a chosen pre-configured Telnet console command.
        """

        cmd = self.uiTelnetConsoleCommandLineEdit.text()
        (ok, cmd) = ConsoleCommandDialog.getCommand(self, console_type="telnet", current=cmd)
        if ok:
            self.uiTelnetConsoleCommandLineEdit.setText(cmd)

    def _vncConsolePreconfiguredCommandSlot(self):
        """
        Slot to set a chosen pre-configured VNC console command.
        """

        cmd = self.uiVNCConsoleCommandLineEdit.text()
        (ok, cmd) = ConsoleCommandDialog.getCommand(self, console_type="vnc", current=cmd)
        if ok:
            self.uiVNCConsoleCommandLineEdit.setText(cmd)

    def _spiceConsolePreconfiguredCommandSlot(self):
        """
        Slot to set a chosen pre-configured SPICE console command.
        """

        cmd = self.uiSPICEConsoleCommandLineEdit.text()
        (ok, cmd) = ConsoleCommandDialog.getCommand(self, console_type="spice", current=cmd)
        if ok:
            self.uiSPICEConsoleCommandLineEdit.setText(cmd)

    def _importConfigurationFileSlot(self):
        """
        Slot to import a configuration file.
        """

        configuration_file_path = LocalConfig.instance().configFilePath()
        directory = os.path.dirname(configuration_file_path)

        path, _ = QtWidgets.QFileDialog.getOpenFileName(self, "Import configuration file", directory, "Configuration file (*.ini *.conf);;All files (*.*)")
        if not path:
            return

        try:
            with open(path, encoding="utf-8") as f:
                config_file = json.load(f)
            if "type" not in config_file or config_file["type"] != "settings":
                QtWidgets.QMessageBox.critical(self, "Import configuration file", "Not a GNS3 configuration file: {}".format(path))
                return
        except OSError as e:
            QtWidgets.QMessageBox.critical(self, "Import configuration file", "Could not load configuration file {}: {}".format(os.path.basename(path), e))
            return
        except (ValueError, TypeError) as e:
            QtWidgets.QMessageBox.critical(self, "Import configuration file", "Invalid file: {}".format(e))
            return

        try:
            shutil.copyfile(path, configuration_file_path)
        except (shutil.Error, IOError) as e:
            QtWidgets.QMessageBox.critical(self, "Import configuration file", "Cannot import configuration file: {}".format(e))
            return

        QtWidgets.QMessageBox.information(self, "Configuration file", "Configuration file imported, default settings will be applied after a restart")

        # TODO: implement restart
        # QtCore.QProcess.startDetached(QtWidgets.QApplication.arguments()[0], QtWidgets.QApplication.arguments())
        # QtWidgets.QApplication.quit()
        LocalConfig.instance().setConfigFilePath(configuration_file_path)
        self._preferences_dialog.reject()

    def _browseConfigurationDirectorySlot(self):
        """
        Slot to open a file browser into the configuration directory
        """
        QtGui.QDesktopServices.openUrl(QtCore.QUrl("file://" + LocalConfig.instance().configDirectory()))

    def _exportConfigurationFileSlot(self):
        """
        Slot to export a configuration file.
        """

        configuration_file_path = LocalConfig.instance().configFilePath()
        directory = os.path.dirname(configuration_file_path)

        path, _ = QtWidgets.QFileDialog.getSaveFileName(self, "Export configuration file", directory, "Configuration file (*.ini *.conf);;All files (*.*)")
        if not path:
            return

        try:
            shutil.copyfile(configuration_file_path, path)
        except (shutil.Error, IOError) as e:
            QtWidgets.QMessageBox.critical(self, "Export configuration file", "Cannot export configuration file: {}".format(e))
            return

    def _setDefaultLabelFontSlot(self):
        """
        Slot to select the default label font.
        """

        selected_font, ok = QtWidgets.QFontDialog.getFont(self.uiDefaultLabelStylePlainTextEdit.font(), self)
        if ok:
            self.uiDefaultLabelStylePlainTextEdit.setFont(selected_font)

    def _setDefaultLabelColorSlot(self):
        """
        Slot to select the default label color.
        """

        color = QtWidgets.QColorDialog.getColor(self._default_label_color, self)
        if color.isValid():
            self._default_label_color = color
            self.uiDefaultLabelStylePlainTextEdit.setStyleSheet("color : {}".format(color.name()))

    def _populateGeneralSettingWidgets(self, settings):
        """
        Populates the widgets with the settings.

        :param settings: General settings
        """

        local_server = LocalServer.instance().localServerSettings()
        self.uiProjectsPathLineEdit.setText(local_server["projects_path"])
        self.uiSymbolsPathLineEdit.setText(local_server["symbols_path"])
        self.uiImagesPathLineEdit.setText(local_server["images_path"])
        self.uiConfigsPathLineEdit.setText(local_server["configs_path"])
        self.uiAppliancesPathLineEdit.setText(local_server["appliances_path"])
        self.uiStatsCheckBox.setChecked(settings["send_stats"])
        self.uiOverlayNotificationsCheckBox.setChecked(settings["overlay_notifications"])
        self.uiCrashReportCheckBox.setChecked(local_server["report_errors"])
        self.uiCheckForUpdateCheckBox.setChecked(settings["check_for_update"])
        self.uiExperimentalFeaturesCheckBox.setChecked(settings["experimental_features"])
        self.uiHdpiCheckBox.setChecked(settings["hdpi"])
        self.uiTelnetConsoleCommandLineEdit.setText(settings["telnet_console_command"])
        self.uiTelnetConsoleCommandLineEdit.setCursorPosition(0)
        index = self.uiStyleComboBox.findText(settings["style"])
        if index != -1:
            self.uiStyleComboBox.setCurrentIndex(index)
        self.uiDelayConsoleAllSpinBox.setValue(settings["delay_console_all"])

        self.uiVNCConsoleCommandLineEdit.setText(settings["vnc_console_command"])
        self.uiVNCConsoleCommandLineEdit.setCursorPosition(0)

        self.uiSPICEConsoleCommandLineEdit.setText(settings["spice_console_command"])
        self.uiSPICEConsoleCommandLineEdit.setCursorPosition(0)

        self.uiMultiProfilesCheckBox.setChecked(settings["multi_profiles"])

        self.uiImageDirectoriesListWidget.clear()
        for path in local_server["additional_images_paths"].split(";"):
            if len(path) > 0:
                self.uiImageDirectoriesListWidget.addItem(path)

        self.uiDirectFileUpload.setChecked(settings["direct_file_upload"])

    def _populateGraphicsViewSettingWidgets(self, settings):
        """
        Populates the widgets with the settings.

        :param settings: Graphics view settings
        """

        self.uiSceneWidthSpinBox.setValue(settings["scene_width"])
        self.uiSceneHeightSpinBox.setValue(settings["scene_height"])
        self.uiGridSizeSpinBox.setValue(settings["grid_size"])
        self.uiRectangleSelectedItemCheckBox.setChecked(settings["draw_rectangle_selected_item"])
        self.uiDrawLinkStatusPointsCheckBox.setChecked(settings["draw_link_status_points"])
        self.uiShowInterfaceLabelsOnNewProject.setChecked(settings["show_interface_labels_on_new_project"])

        qt_font = QtGui.QFont()
        if qt_font.fromString(settings["default_label_font"]):
            self.uiDefaultLabelStylePlainTextEdit.setFont(qt_font)
        qt_color = QtGui.QColor(settings["default_label_color"])
        if qt_color.isValid():
            self._default_label_color = qt_color
            self.uiDefaultLabelStylePlainTextEdit.setStyleSheet("color : {}".format(qt_color.name()))

    def loadPreferences(self):
        """
        Loads the general preferences.
        """

        from ..main_window import MainWindow
        general_settings = MainWindow.instance().settings()
        self._populateGeneralSettingWidgets(general_settings)

        graphics_view_settings = MainWindow.instance().uiGraphicsView.settings()
        self._populateGraphicsViewSettingWidgets(graphics_view_settings)

    def savePreferences(self):
        """
        Saves the general preferences.
        """

        additional_images_paths = set()
        for i in range(0, self.uiImageDirectoriesListWidget.count()):
            item = self.uiImageDirectoriesListWidget.item(i)
            additional_images_paths.add(item.text())

        new_local_server_settings = {"images_path": self.uiImagesPathLineEdit.text(),
                                     "projects_path": self.uiProjectsPathLineEdit.text(),
                                     "symbols_path": self.uiSymbolsPathLineEdit.text(),
                                     "configs_path": self.uiConfigsPathLineEdit.text(),
                                     "appliances_path": self.uiAppliancesPathLineEdit.text(),
                                     "report_errors": self.uiCrashReportCheckBox.isChecked(),
                                     "additional_images_paths": ":".join(additional_images_paths)}
        LocalServer.instance().updateLocalServerSettings(new_local_server_settings)

        new_general_settings = {
            "style": self.uiStyleComboBox.currentText(),
            "experimental_features": self.uiExperimentalFeaturesCheckBox.isChecked(),
            "hdpi": self.uiHdpiCheckBox.isChecked(),
            "check_for_update": self.uiCheckForUpdateCheckBox.isChecked(),
            "overlay_notifications": self.uiOverlayNotificationsCheckBox.isChecked(),
            "telnet_console_command": self.uiTelnetConsoleCommandLineEdit.text(),
            "vnc_console_command": self.uiVNCConsoleCommandLineEdit.text(),
            "spice_console_command": self.uiSPICEConsoleCommandLineEdit.text(),
            "delay_console_all": self.uiDelayConsoleAllSpinBox.value(),
            "send_stats": self.uiStatsCheckBox.isChecked(),
            "multi_profiles": self.uiMultiProfilesCheckBox.isChecked(),
            "direct_file_upload": self.uiDirectFileUpload.isChecked()
        }

        from ..main_window import MainWindow
        MainWindow.instance().setSettings(new_general_settings)

        new_graphics_view_settings = {"scene_width": self.uiSceneWidthSpinBox.value(),
                                      "scene_height": self.uiSceneHeightSpinBox.value(),
                                      "grid_size": self.uiGridSizeSpinBox.value(),
                                      "draw_rectangle_selected_item": self.uiRectangleSelectedItemCheckBox.isChecked(),
                                      "draw_link_status_points": self.uiDrawLinkStatusPointsCheckBox.isChecked(),
                                      "show_interface_labels_on_new_project": self.uiShowInterfaceLabelsOnNewProject.isChecked(),
                                      "default_label_font": self.uiDefaultLabelStylePlainTextEdit.font().toString(),
                                      "default_label_color": self._default_label_color.name()}
        MainWindow.instance().uiGraphicsView.setSettings(new_graphics_view_settings)



# -*- coding: utf-8 -*-
#
# Copyright (C) 2014 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
QEMU module implementation.
"""

import sys

from gns3.qt import QtWidgets
from gns3.local_config import LocalConfig
from gns3.local_server_config import LocalServerConfig

from ...controller import Controller
from ..module import Module
from ..module_error import ModuleError
from .qemu_vm import QemuVM
from .settings import QEMU_SETTINGS
from .settings import QEMU_VM_SETTINGS

import logging
log = logging.getLogger(__name__)


class Qemu(Module):

    """
    QEMU module.
    """

    def __init__(self):
        super().__init__()

        self._settings = {}
        self._qemu_vms = {}
        self._nodes = []

        self.configChangedSlot()

    def configChangedSlot(self):
        # load the settings
        self._loadSettings()

    def _loadSettings(self):
        """
        Loads the settings from the persistent settings file.
        """

        self._settings = LocalConfig.instance().loadSectionSettings(self.__class__.__name__, QEMU_SETTINGS)
        self._loadQemuVMs()

    def _saveSettings(self):
        """
        Saves the settings to the persistent settings file.
        """

        # save the settings
        LocalConfig.instance().saveSectionSettings(self.__class__.__name__, self._settings)

        # save some settings to the local server config file
        if sys.platform.startswith("linux"):
            server_settings = {
                "enable_kvm": self._settings["enable_kvm"],
                "require_kvm": self._settings["require_kvm"]
            }

            LocalServerConfig.instance().saveSettings(self.__class__.__name__, server_settings)

    def _loadQemuVMs(self):
        """
        Load the QEMU VMs from the persistent settings file.
        """

        self._qemu_vms = {}
        settings = LocalConfig.instance().settings()
        if "vms" in settings.get(self.__class__.__name__, {}):
            for vm in settings[self.__class__.__name__]["vms"]:
                name = vm.get("name")
                server = vm.get("server")
                key = "{server}:{name}".format(server=server, name=name)
                if key in self._qemu_vms or not name or not server:
                    continue
                vm_settings = QEMU_VM_SETTINGS.copy()
                vm_settings.update(vm)
                # for backward compatibility before version 1.4
                if "symbol" not in vm_settings:
                    vm_settings["symbol"] = vm_settings.get("default_symbol", vm_settings["symbol"])
                    vm_settings["symbol"] = vm_settings["symbol"][:-11] + ".svg" if vm_settings["symbol"].endswith("normal.svg") else vm_settings["symbol"]
                self._qemu_vms[key] = vm_settings

    def _saveQemuVMs(self):
        """
        Saves the QEMU VMs to the persistent settings file.
        """

        self._settings["vms"] = list(self._qemu_vms.values())
        self._saveSettings()

    def VMs(self):
        """
        Returns QEMU VMs settings.

        :returns: QEMU VMs settings (dictionary)
        """

        return self._qemu_vms

    def setVMs(self, new_qemu_vms):
        """
        Sets QEMU VM settings.

        :param new_qemu_vms: Qemu images settings (dictionary)
        """

        self._qemu_vms = new_qemu_vms.copy()
        self._saveQemuVMs()

    def addNode(self, node):
        """
        Adds a node to this module.

        :param node: Node instance
        """

        self._nodes.append(node)

    def removeNode(self, node):
        """
        Removes a node from this module.

        :param node: Node instance
        """

        if node in self._nodes:
            self._nodes.remove(node)

    def settings(self):
        """
        Returns the module settings

        :returns: module settings (dictionary)
        """

        return self._settings

    def setSettings(self, settings):
        """
        Sets the module settings

        :param settings: module settings (dictionary)
        """

        self._settings.update(settings)
        self._saveSettings()

    def instantiateNode(self, node_class, server, project):
        """
        Instantiate a new node.

        :param node_class: Node object
        :param server: HTTPClient instance
        """

        # create an instance of the node class
        return node_class(self, server, project)

    def reset(self):
        """
        Resets the servers.
        """

        self._nodes.clear()

    def getQemuBinariesFromServer(self, compute_id, callback, archs=None):
        """
        Gets the QEMU binaries list from a server.

        :param compute_id: server to send the request to
        :param callback: callback for the reply from the server
        :param archs: A list of architectures. Only binaries matching the specified architectures are returned.
        """

        request_body = None
        if archs is not None:
            request_body = {"archs": archs}
        Controller.instance().getCompute("/qemu/binaries", compute_id, callback, body=request_body)

    def getQemuImgBinariesFromServer(self, compute_id, callback):
        """
        Gets the QEMU-img binaries list from a server.

        :param server: server to send the request to
        :param callback: callback for the reply from the server
        """

        Controller.instance().getCompute("/qemu/img-binaries", compute_id, callback)

    def getQemuCapabilitiesFromServer(self, compute_id, callback):
        """
        Gets the capabilities of Qemu at a server.

        :param server: server to send the request to
        :param callback: callback for the reply from the server
        """

        Controller.instance().getCompute("/qemu/capabilities", compute_id, callback)

    def createDiskImage(self, compute_id, callback, options):
        """
        Create a disk image on the remote server

        :param server: server to send the request to
        :param callback: callback for the reply from the server
        :param options: Options for the image creation
        """

        Controller.instance().postCompute("/qemu/img", compute_id, callback, body=options)

    @staticmethod
    def getNodeClass(name):
        """
        Returns the object with the corresponding name.

        :param name: object name
        """

        if name in globals():
            return globals()[name]
        return None

    @staticmethod
    def getNodeType(name, platform=None):
        if name == "qemu":
            return QemuVM
        return None

    @staticmethod
    def classes():
        """
        Returns all the node classes supported by this module.

        :returns: list of classes
        """

        return [QemuVM]

    def nodes(self):
        """
        Returns all the node data necessary to represent a node
        in the nodes view and create a node on the scene.
        """

        nodes = []
        for qemu_vm in self._qemu_vms.values():
            nodes.append(
                {"class": QemuVM.__name__,
                 "name": qemu_vm["name"],
                 "server": qemu_vm["server"],
                 "symbol": qemu_vm["symbol"],
                 "categories": [qemu_vm["category"]]
                 }
            )
        return nodes

    @staticmethod
    def preferencePages():
        """
        :returns: QWidget object list
        """

        from .pages.qemu_preferences_page import QemuPreferencesPage
        from .pages.qemu_vm_preferences_page import QemuVMPreferencesPage
        return [QemuPreferencesPage, QemuVMPreferencesPage]

    @staticmethod
    def vmConfigurationPage():
        from .pages.qemu_vm_configuration_page import QemuVMConfigurationPage
        return QemuVMConfigurationPage

    @staticmethod
    def instance():
        """
        Singleton to return only one instance of QEMU module.

        :returns: instance of Qemu
        """

        if not hasattr(Qemu, "_instance"):
            Qemu._instance = Qemu()
        return Qemu._instance



# -*- coding: utf-8 -*-
#
# Copyright (C) 2014 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
VPCS module implementation.
"""

import os
import copy
import shutil

from gns3.local_config import LocalConfig
from gns3.local_server_config import LocalServerConfig

from ..module import Module
from .vpcs_node import VPCSNode
from .settings import VPCS_SETTINGS
from .settings import VPCS_NODES_SETTINGS

import logging
log = logging.getLogger(__name__)


class VPCS(Module):

    """
    VPCS module.
    """

    def __init__(self):
        super().__init__()

        self._settings = {}
        self._nodes = []
        self._vpcs_nodes = {}
        self._working_dir = ""
        self._loadSettings()

    def configChangedSlot(self):
        # load the settings
        self._loadSettings()

    def _loadSettings(self):
        """
        Loads the settings from the persistent settings file.
        """

        self._settings = LocalConfig.instance().loadSectionSettings(self.__class__.__name__, VPCS_SETTINGS)
        if not os.path.exists(self._settings["vpcs_path"]):
            vpcs_path = shutil.which("vpcs")
            if vpcs_path:
                self._settings["vpcs_path"] = os.path.abspath(vpcs_path)
            else:
                self._settings["vpcs_path"] = ""

        self._loadVPCSNodes()

    def _saveSettings(self):
        """
        Saves the settings to the persistent settings file.
        """

        # save the settings
        LocalConfig.instance().saveSectionSettings(self.__class__.__name__, self._settings)

        server_settings = {}
        if self._settings["vpcs_path"]:
            # save some settings to the server config file
            server_settings["vpcs_path"] = os.path.normpath(self._settings["vpcs_path"])
        config = LocalServerConfig.instance()
        config.saveSettings(self.__class__.__name__, server_settings)

    def _loadVPCSNodes(self):
        """
        Load the VPCS nodes from the persistent settings file.
        """

        self._vpcs_nodes = {}
        settings = LocalConfig.instance().settings()
        if "nodes" in settings.get(self.__class__.__name__, {}):
            for node in settings[self.__class__.__name__]["nodes"]:
                name = node.get("name")
                server = node.get("server")
                key = "{server}:{name}".format(server=server, name=name)
                if key in self._vpcs_nodes or not name or not server:
                    continue
                node_settings = VPCS_NODES_SETTINGS.copy()
                node_settings.update(node)
                self._vpcs_nodes[key] = node_settings

    def _saveVPCSNodes(self):
        """
        Saves the VPCS nodes to the persistent settings file.
        """

        self._settings["nodes"] = list(self._vpcs_nodes.values())
        self._saveSettings()

    def addNode(self, node):
        """
        Adds a node to this module.

        :param node: Node instance
        """

        self._nodes.append(node)

    def removeNode(self, node):
        """
        Removes a node from this module.

        :param node: Node instance
        """

        if node in self._nodes:
            self._nodes.remove(node)

    def settings(self):
        """
        Returns the module settings

        :returns: module settings (dictionary)
        """

        return self._settings

    def setSettings(self, settings):
        """
        Sets the module settings

        :param settings: module settings (dictionary)
        """

        self._settings.update(settings)
        self._saveSettings()

    def instantiateNode(self, node_class, server, project):
        """
        Instantiate a new node.

        :param node_class: Node object
        :param server: HTTPClient instance
        :param project: Project instance
        """

        # create an instance of the node class
        return node_class(self, server, project)

    def reset(self):
        """
        Resets the module.
        """

        self._nodes.clear()

    @staticmethod
    def getNodeType(name, platform=None):
        if name == "vpcs":
            return VPCSNode
        return None

    @staticmethod
    def vmConfigurationPage():
        from .pages.vpcs_node_configuration_page import VPCSNodeConfigurationPage
        return VPCSNodeConfigurationPage

    def VMs(self):
        """
        Returns list of VPCS nodes
        """

        return self._vpcs_nodes

    def setVMs(self, new_vpcs_nodes):
        """
        Sets VPCS list

        :param new_vpcs_vms: VPCS node list
        """

        self._vpcs_nodes = new_vpcs_nodes.copy()
        self._saveVPCSNodes()

    @staticmethod
    def classes():
        """
        Returns all the node classes supported by this module.

        :returns: list of classes
        """

        return [VPCSNode]

    def nodes(self):
        """
        Returns all the node data necessary to represent a node
        in the nodes view and create a node on the scene.
        """

        nodes = []

        # Add a default VPCS not linked to a specific server
        nodes.append(
            {
                "class": VPCSNode.__name__,
                "name": "VPCS",
                "categories": [VPCSNode.end_devices],
                "symbol": VPCSNode.defaultSymbol(),
                "builtin": True
            }
        )
        return nodes

    @staticmethod
    def preferencePages():
        """
        :returns: QWidget object list
        """

        from .pages.vpcs_preferences_page import VPCSPreferencesPage
        from .pages.vpcs_node_preferences_page import VPCSNodePreferencesPage
        return [VPCSPreferencesPage, VPCSNodePreferencesPage]

    @staticmethod
    def instance():
        """
        Singleton to return only on instance of VPCS module.

        :returns: instance of VPCS
        """

        if not hasattr(VPCS, "_instance"):
            VPCS._instance = VPCS()
        return VPCS._instance



# -*- coding: utf-8 -*-
#
# Copyright (C) 2015 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
Configuration page for VMware preferences.
"""

import os
import sys
import shutil
from gns3.qt import QtWidgets, qpartial
from gns3.utils.sudo import sudo
from gns3.controller import Controller

from .. import VMware
from ..ui.vmware_preferences_page_ui import Ui_VMwarePreferencesPageWidget
from ..settings import VMWARE_SETTINGS


class VMwarePreferencesPage(QtWidgets.QWidget, Ui_VMwarePreferencesPageWidget):

    """
    QWidget preference page for VMware.
    """

    def __init__(self):

        super().__init__()
        self.setupUi(self)

        # connect signals
        self.uiRestoreDefaultsPushButton.clicked.connect(self._restoreDefaultsSlot)
        self.uiVmrunPathToolButton.clicked.connect(self._vmrunPathBrowserSlot)
        self.uiConfigureVmnetPushButton.clicked.connect(self._configureVmnetSlot)
        self.uiResetVmnetPushButton.clicked.connect(self._resetVmnetSlot)

        if sys.platform.startswith("win"):
            # VMnet limit on Windows is 19
            self.uiVMnetEndRangeSpinBox.setMaximum(19)
        else:
            # VMnet limit on Linux is 255
            self.uiVMnetEndRangeSpinBox.setMaximum(255)

    def _vmrunPathBrowserSlot(self):
        """
        Slot to open a file browser and select vmrun.
        """

        vmrun_path = shutil.which("vmrun")
        path, _ = QtWidgets.QFileDialog.getOpenFileName(self, "Select vmrun", vmrun_path)
        if not path:
            return

        if self._checkVmrunPath(path):
            self.uiVmrunPathLineEdit.setText(os.path.normpath(path))

    def _checkVmrunPath(self, path):
        """
        Checks that the vmrun path is valid.

        :param path: vmrun path
        :returns: boolean
        """

        if not os.path.exists(path):
            QtWidgets.QMessageBox.critical(self, "vmrun", '"{}" does not exist'.format(path))
            return False

        if not os.access(path, os.X_OK):
            QtWidgets.QMessageBox.critical(self, "vmrun", "{} is not an executable".format(os.path.basename(path)))
            return False

        return True

    def _restoreDefaultsSlot(self):
        """
        Slot to populate the page widgets with the default settings.
        """

        self._populateWidgets(VMWARE_SETTINGS)

    def _useLocalServerSlot(self, state):
        """
        Slot to enable or not local server settings.
        """

        if state:
            self.uiVmrunPathLineEdit.setEnabled(True)
            self.uiVmrunPathToolButton.setEnabled(True)
            self.uiNetworkTab.setEnabled(True)
        else:
            self.uiVmrunPathLineEdit.setEnabled(False)
            self.uiVmrunPathToolButton.setEnabled(False)
            self.uiNetworkTab.setEnabled(False)

    def _getGNS3Vmnet(self):
        """
        Get the gns3vmnet utility path.
        """

        gns3vmnet = shutil.which("gns3vmnet")
        if gns3vmnet is None:
            QtWidgets.QMessageBox.critical(self, "gns3vmnet", "The gns3vmnet utility is not installed")
            return None
        return gns3vmnet

    def _configureVmnetSlot(self):
        """
        Configure the vmnet interfaces.
        """

        vmnet_start = str(self.uiVMnetStartRangeSpinBox.value())
        vmnet_end = str(self.uiVMnetEndRangeSpinBox.value())
        gns3vmnet = self._getGNS3Vmnet()
        if gns3vmnet is None:
            return
        command = [gns3vmnet, "-r", vmnet_start, vmnet_end]
        sudo(command, parent=self)

    def _resetVmnetSlot(self):
        """
        Deletes all vmnet interface but vmnet1 and vmnet8
        """

        gns3vmnet = self._getGNS3Vmnet()
        if gns3vmnet is None:
            return
        command = [gns3vmnet, "-C"]
        sudo(command, parent=self)

    def _populateWidgets(self, settings):
        """
        Populates the widgets with the settings.

        :param settings: VMware settings
        """

        self.uiVmrunPathLineEdit.setText(settings["vmrun_path"])
        self.uiVMnetStartRangeSpinBox.setValue(settings["vmnet_start_range"])
        self.uiVMnetEndRangeSpinBox.setValue(settings["vmnet_end_range"])
        self.uiBlockHostTrafficCheckBox.setChecked(settings["block_host_traffic"])

    def loadPreferences(self):
        """
        Loads VMware preferences.
        """

        vmware_settings = VMware.instance().settings()
        self._populateWidgets(vmware_settings)

    def savePreferences(self):
        """
        Saves VMware preferences.
        """

        vmrun_path = self.uiVmrunPathLineEdit.text().strip()
        if vmrun_path and not self._checkVmrunPath(vmrun_path):
            return

        new_settings = {"vmrun_path": vmrun_path,
                        "vmnet_start_range": self.uiVMnetStartRangeSpinBox.value(),
                        "vmnet_end_range": self.uiVMnetEndRangeSpinBox.value(),
                        "block_host_traffic": self.uiBlockHostTrafficCheckBox.isChecked()}
        VMware.instance().setSettings(new_settings)



# -*- coding: utf-8 -*-
#
# Copyright (C) 2013 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
Manages and stores everything needed for a connection between 2 devices.
"""

import os
import re
import sip
import uuid

from .qt import QtCore, QtWidgets
from .controller import Controller


import logging
log = logging.getLogger(__name__)


class Link(QtCore.QObject):

    """
    Link implementation.

    :param source_node: source Node instance
    :param source_port: source Port instance
    :param destination_node: destination Node instance
    :param destination_port: destination Port instance
    """

    # signals used to let the GUI view know about link
    # additions and deletions.
    add_link_signal = QtCore.Signal(int)
    delete_link_signal = QtCore.Signal(int)
    updated_link_signal = QtCore.Signal(int)
    error_link_signal = QtCore.Signal(int)

    _instance_count = 1

    def __init__(self, source_node, source_port, destination_node, destination_port, link_id=None, **link_data):
        """
        :param link_data: Link information from the API
        """

        super().__init__()

        log.debug("adding link from {} {} to {} {}".format(source_node.name(),
                                                           source_port.name(),
                                                           destination_node.name(),
                                                           destination_port.name()))

        # create an unique ID
        self._id = Link._instance_count
        Link._instance_count += 1

        self._source_node = source_node
        self._source_port = source_port
        self._destination_node = destination_node
        self._destination_port = destination_port
        self._source_label = None
        self._destination_label = None
        self._link_id = link_id
        self._capturing = False
        self._deleting = False
        self._capture_file_path = None
        self._capture_file = None
        self._initialized = False
        self._filters = {}
        self._suspend = False

        # Boolean if True we are creating the first instance of this node
        # if false the node already exist in the topology
        # use to avoid erasing information when reloading
        self._creator = False

        self._nodes = []

        self._source_node.addLink(self)
        self._destination_node.addLink(self)

        body = self._prepareParams()
        if self._link_id:
            link_data["link_id"] = self._link_id
            self._linkCreatedCallback(link_data)
        else:
            self._link_id = str(uuid.uuid4())
            self._creator = True
            Controller.instance().post("/projects/{project_id}/links".format(project_id=source_node.project().id()), self._linkCreatedCallback, body=body)

    def _parseResponse(self, result):
        self._capturing = result.get("capturing", False)

        # If the controller is remote the capture path should be rewrite to something local
        if self._capturing:
            if Controller.instance().isRemote():
                if self._capture_file_path is None and result.get("capture_file_path", None) is not None:
                    self._capture_file = QtCore.QTemporaryFile()
                    self._capture_file.open(QtCore.QFile.WriteOnly)
                    self._capture_file.setAutoRemove(True)
                    self._capture_file_path = self._capture_file.fileName()
                    Controller.instance().get(
                        "/projects/{project_id}/links/{link_id}/pcap".format(
                            project_id=self.project().id(),
                            link_id=self._link_id),
                        None,
                        showProgress=False,
                        downloadProgressCallback=self._downloadPcapProgress,
                        ignoreErrors=True,  # If something is wrong avoid disconnect us from server
                        timeout=None)
            else:
                self._capture_file_path = result["capture_file_path"]

        if "nodes" in result:
            self._nodes = result["nodes"]
            self._updateLabels()
        if "filters" in result:
            self._filters = result["filters"]
        if "suspend" in result:
            self._suspend = result["suspend"]
        self.updated_link_signal.emit(self._id)

    def creator(self):
        return self._creator

    def suspended(self):
        return self._suspend

    def toggleSuspend(self):
        self._suspend = not self._suspend
        self.update()

    def initialized(self):
        return self._initialized

    def addPortLabel(self, port, label):
        if port.adapterNumber() == self._source_port.adapterNumber() and port.portNumber() == self._source_port.portNumber() and port.destinationNode() == self._destination_node:
            self._source_label = label
        else:
            self._destination_label = label
        label.item_unselected_signal.connect(self.update)
        if self.creator():
            self.update()
        else:
            self._updateLabels()

    def update(self):
        if not self._link_id or self.deleting():
            return
        body = self._prepareParams()
        Controller.instance().put("/projects/{project_id}/links/{link_id}".format(project_id=self._source_node.project().id(), link_id=self._link_id), self.updateLinkCallback, body=body)

    def listAvailableFilters(self, callback):
        """
        Get the list of available filters
        """
        Controller.instance().get("/projects/{project_id}/links/{link_id}/available_filters".format(project_id=self._source_node.project().id(), link_id=self._link_id), callback)

    def updateLinkCallback(self, result, error=False, *args, **kwargs):
        if error:
            QtWidgets.QMessageBox.warning(None, "Update link", "Error while updating link: {}".format(result["message"]))
            return
        self._parseResponse(result)

    def _updateLabels(self):
        for node in self._nodes:
            if node["node_id"] == self._source_node.node_id() and node["adapter_number"] == self._source_port.adapterNumber() and node["port_number"] == self._source_port.portNumber():
                self._updateLabel(self._source_label, node["label"])
            elif node["node_id"] == self._destination_node.node_id() and node["adapter_number"] == self._destination_port.adapterNumber() and node["port_number"] == self._destination_port.portNumber():
                self._updateLabel(self._destination_label, node["label"])
            else:
                raise NotImplementedError

    def _updateLabel(self, label, label_data):
        if not label or sip.isdeleted(label):
            return
        if "text" in label_data:
            label.setPlainText(label_data["text"])
        if "x" in label_data and "y" in label_data:
            label.setPos(label_data["x"], label_data["y"])
        if "style" in label_data:
            label.setStyle(label_data["style"])
        if "rotation" in label_data:
            label.setRotation(label_data["rotation"])

    def _prepareParams(self):
        body = {
            "nodes": [
                {
                    "node_id": self._source_node.node_id(),
                    "adapter_number": self._source_port.adapterNumber(),
                    "port_number": self._source_port.portNumber(),
                },
                {
                    "node_id": self._destination_node.node_id(),
                    "adapter_number": self._destination_port.adapterNumber(),
                    "port_number": self._destination_port.portNumber()
                }
            ],
            "filters": self._filters,
            "suspend": self._suspend
        }
        if self._source_port.label():
            body["nodes"][0]["label"] = self._source_port.label().dump()
        if self._destination_port.label():
            body["nodes"][1]["label"] = self._destination_port.label().dump()
        return body

    def _linkCreatedCallback(self, result, error=False, **kwargs):
        if error:
            QtWidgets.QMessageBox.warning(None, "Create link", "Error while creating link: {}".format(result["message"]))
            self.deleteLink(skip_controller=True)
            return

        self._initialized = True

        # let the GUI know about this link has been created
        self.add_link_signal.emit(self._id)
        self._source_port.setLinkId(self._id)
        self._source_port.setLink(self)
        self._source_port.setDestinationNode(self._destination_node)
        self._source_port.setDestinationPort(self._destination_port)
        self._destination_port.setLinkId(self._id)
        self._destination_port.setLink(self)
        self._destination_port.setDestinationNode(self._source_node)
        self._destination_port.setDestinationPort(self._source_port)

        self._link_id = result["link_id"]
        self._parseResponse(result)

    def link_id(self):
        return self._link_id

    def deleting(self):
        """
        Is the link being deleted
        """
        return self._deleting

    def setDeleting(self):
        """
        Mark this link as being deleted
        """

        self._deleting = True

    def capturing(self):
        """
        Is a capture running on the link?
        """
        return self._capturing

    def capture_file_path(self):
        """
        Path of the capture file
        """
        return self._capture_file_path

    def project(self):
        return self._source_node.project()

    @classmethod
    def reset(cls):
        """
        Reset the instance count.
        """

        cls._instance_count = 1

    def __str__(self):

        description = "Link from {} port {} to {} port {}".format(self._source_node.name(),
                                                                  self._source_port.name(),
                                                                  self._destination_node.name(),
                                                                  self._destination_port.name())

        if self.capturing():
            description += "\nPacket capture is active"

        for filter_type in self._filters.keys():
            description += "\nPacket filter '{}' is active".format(filter_type)

        return description

    def capture_file_name(self):
        """
        :returns: File name for a capture on this link
        """
        capture_file_name = "{}_{}_to_{}_{}".format(
            self._source_node.name(),
            self._source_port.name(),
            self._destination_node.name(),
            self._destination_port.name())
        return re.sub("[^0-9A-Za-z_-]", "", capture_file_name)

    def deleteLink(self, skip_controller=False):
        """
        Deletes this link.
        """

        log.debug("deleting link from {} {} to {} {}".format(self._source_node.name(),
                                                             self._source_port.name(),
                                                             self._destination_node.name(),
                                                             self._destination_port.name()))

        if skip_controller:
            self._linkDeletedCallback({})
        else:
            self.setDeleting()
            Controller.instance().delete("/projects/{project_id}/links/{link_id}".format(project_id=self.project().id(),
                                                                                         link_id=self._link_id),
                                                                                         self._linkDeletedCallback)

    def _linkDeletedCallback(self, result, error=False, **kwargs):
        """
        Called after the link is remove from the topology
        """
        if error:
            log.error("Error while deleting link: {}".format(result["message"]))
            return

        self._source_port.setFree()
        self._source_node.deleteLink(self)
        self._source_node.updated_signal.emit()
        self._destination_port.setFree()
        self._destination_node.deleteLink(self)
        self._destination_node.updated_signal.emit()

        # let the GUI know about this link has been deleted
        self.delete_link_signal.emit(self._id)

    def startCapture(self, data_link_type, capture_file_name):
        data = {
            "capture_file_name": capture_file_name,
            "data_link_type": data_link_type
        }
        Controller.instance().post(
            "/projects/{project_id}/links/{link_id}/start_capture".format(
                project_id=self.project().id(),
                link_id=self._link_id),
            self._startCaptureCallback,
            body=data)

    def _startCaptureCallback(self, result, error=False, **kwargs):
        if error:
            log.error("Error while starting capture on link: {}".format(result["message"]))
            return
        self._parseResponse(result)

    def _downloadPcapProgress(self, content, server=None, context={}, **kwargs):
        """
        Called for each part of the file of the PCAP
        """
        if not self._capture_file_path:
            return
        self._capture_file.write(content)
        self._capture_file.flush()

    def stopCapture(self):
        if Controller.instance().isRemote():
            if self._capture_file:
                self._capture_file.close()
                self._capture_file = None
            if self._capture_file_path and os.path.exists(self._capture_file_path):
                try:
                    os.remove(self._capture_file_path)
                except OSError as e:
                    log.error("Can't remove file {}: {}".format(self._capture_file_path, e))
        self._capture_file_path = None
        Controller.instance().post(
            "/projects/{project_id}/links/{link_id}/stop_capture".format(
                project_id=self.project().id(),
                link_id=self._link_id),
            self._stopCaptureCallback)

    def _stopCaptureCallback(self, result, error=False, **kwargs):
        if error:
            log.error("Error while stopping capture on link: {}".format(result["message"]))
            return
        self._parseResponse(result)

    def get(self, path, callback, **kwargs):
        """
        HTTP Get from a link
        """
        Controller.instance().get(
            "/projects/{project_id}/links/{link_id}{path}".format(
                project_id=self.project().id(),
                link_id=self._link_id,
                path=path),
            callback,
            **kwargs)

    def id(self):
        """
        Returns this link identifier.

        :returns: link identifier (integer)
        """

        return self._id

    def sourceNode(self):
        """
        Returns the source node for this link.

        :returns: Node instance
        """

        return self._source_node

    def destinationNode(self):
        """
        Returns the destination node for this link.

        :returns: Node instance
        """

        return self._destination_node

    def sourcePort(self):
        """
        Returns the source port for this link.

        :returns: Port instance
        """

        return self._source_port

    def destinationPort(self):
        """
        Returns the destination port for this link.

        :returns: Port instance
        """

        return self._destination_port

    def getNodePort(self, node):
        """
        Search the port in the link corresponding to this node

        :returns: Node instance
        """
        if self._destination_node == node:
            return self._destination_port
        return self._source_port

    def filters(self):
        """
        :returns: List the filters active on the node
        """
        return self._filters

    def setFilters(self, filters):
        """
        :params filters: List of filters
        """
        self._filters = filters



# -*- coding: utf-8 -*-
#
# Copyright (C) 2014 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
Progress dialog that blocking tasks (file operations, network connections etc.)
"""

import sip
from gns3.version import __version__
from ..qt import QtWidgets, QtCore, qslot

import logging
log = logging.getLogger(__name__)


class ProgressDialog(QtWidgets.QProgressDialog):

    """
    Progress dialog implementation with thread support.

    :param thread: thread to run
    :param title: window title
    :param label_text: text to describe the progress bar
    :param cancel_button_text: text for the cancel button
    :param busy: if True, the progress bar in "sliding mode"
    :param delay: Countdown in seconds before starting the worker
    :param create_thread: Start the worker in a dedicated thread
    to show unknown progress.
    :param parent: parent widget
    """

    def __init__(self, worker, title, label_text, cancel_button_text, busy=False, parent=None, delay=0, create_thread=True, cancelable=False):

        if "dev" in __version__:
            assert QtCore.QThread.currentThread() == QtWidgets.QApplication.instance().thread()

        minimum = 0
        maximum = 100

        if busy:
            maximum = 0

        super().__init__(label_text, cancel_button_text, minimum, maximum, parent)
        self.setModal(True)
        self.setAttribute(QtCore.Qt.WA_DeleteOnClose, True)
        self._errors = []
        self.setWindowTitle(title)
        self.canceled.connect(self._canceledSlot)
        self.destroyed.connect(self._cleanup)
        self._cancelable = cancelable
        self._worker = worker
        self._worker.setObjectName(worker.__class__.__name__)
        if create_thread:
            self._thread = QtCore.QThread()
            if "dev" in __version__:
                assert not self._worker.parent()
            log.debug("Set worker inside a thread {}".format(self._worker.__class__))
            self._worker.moveToThread(self._thread)
        else:
            log.debug("Set worker outside of a thread {}".format(self._worker.__class__))
            self._thread = None
        self._worker.finished.connect(self.accept)
        self._worker.updated.connect(self._updateProgressSlot)
        self._worker.error.connect(self._error)
        if self._thread:
            self._thread.started.connect(self._worker.run)

        self._countdownTimer = None
        if delay == 0:
            self._start()
        else:
            self._delay = delay
            self._countdownTimer = QtCore.QTimer()
            self._countdownTimer.setInterval(self._delay * 100)
            self._countdownTimer.timeout.connect(self._updateCountdownSlot)
            self._countdownTimer.start()
            self._updateCountdownSlot()

    @qslot
    def _updateCountdownSlot(self):
        """
        Called every second for countdown before
        starting the worker
        """
        if self._delay <= 0:
            self.setCancelButtonText("Cancel")
            self._countdownTimer.stop()
            self._start()
        else:
            self.setCancelButtonText("Cancel start ({} seconds)".format(self._delay))
            self._delay -= 1

    def _start(self):
        #  connect the thread signals and start the thread

        if self._thread:
            self._thread.start()
            log.debug("{} thread started".format(self._worker.objectName()))
        elif self._worker:
            self._worker.run()

    @qslot
    def _canceledSlot(self):
        if self._cancelable and not self._thread:
            self._worker.cancel()
            log.debug("{} worker canceled".format(self._worker.objectName()))

        if self._thread:
            self._worker.cancel()
            log.debug("{} thread canceled".format(self._worker.objectName()))
        self._cleanup()

    @qslot
    def accept(self):
        if self._worker:
            log.debug("{} thread finished".format(self._worker.objectName()))
        self._cleanup()
        super().accept()

    def __del__(self):

        self._cleanup()

    @qslot
    def _cleanup(self):
        """
        Delete the thread.
        """

        if self._countdownTimer:
            self._countdownTimer.stop()

        if self._thread and not sip.isdeleted(self._thread):
            if self._thread.isRunning():
                log.debug("{} thread is being destroyed".format(self._worker.objectName()))
                thread = self._thread
                self._thread = None
                thread.quit()
                if not thread.wait(3000):
                    thread.terminate()
                    thread.wait()
                log.debug("{} thread destroyed".format(self._worker.objectName()))
                thread.deleteLater()
            self._worker = None

    @qslot
    def _updateProgressSlot(self, value):
        """
        Slot to update the progress bar value.

        :param value: value for the progress bar (integer)
        """

        if self._thread:
            # It seems in some cases this is called on a deleted object and crash
            self.setValue(value)

    @qslot
    def _error(self, message, stop=False):
        """
        Slot to show an error message sent by the thread.

        :param message: message
        """

        if stop:
            log.critical("{} thread stopping with an error: {}".format(self._worker.objectName(), message))
            self._canceledSlot()
            QtWidgets.QMessageBox.critical(self.parentWidget(), "Error", "{}".format(message))
        else:
            self._errors.append(message)
            self._cleanup()

    def errors(self):
        """
        Returns error messages.

        :returns: error message list
        """

        return self._errors



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '/home/grossmj/PycharmProjects/gns3-gui/gns3/modules/traceng/ui/traceng_preferences_page.ui'
#
# Created by: PyQt5 UI code generator 5.5.1
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_TraceNGPreferencesPageWidget(object):
    def setupUi(self, TraceNGPreferencesPageWidget):
        TraceNGPreferencesPageWidget.setObjectName("TraceNGPreferencesPageWidget")
        TraceNGPreferencesPageWidget.resize(623, 280)
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(TraceNGPreferencesPageWidget)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.uiTabWidget = QtWidgets.QTabWidget(TraceNGPreferencesPageWidget)
        self.uiTabWidget.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
        self.uiTabWidget.setObjectName("uiTabWidget")
        self.uiGeneralSettingsTabWidget = QtWidgets.QWidget()
        self.uiGeneralSettingsTabWidget.setObjectName("uiGeneralSettingsTabWidget")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.uiGeneralSettingsTabWidget)
        self.verticalLayout.setContentsMargins(10, 10, 10, 10)
        self.verticalLayout.setObjectName("verticalLayout")
        self.uiTraceNGPathLabel = QtWidgets.QLabel(self.uiGeneralSettingsTabWidget)
        self.uiTraceNGPathLabel.setObjectName("uiTraceNGPathLabel")
        self.verticalLayout.addWidget(self.uiTraceNGPathLabel)
        self.horizontalLayout_5 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_5.setObjectName("horizontalLayout_5")
        self.uiTraceNGPathLineEdit = QtWidgets.QLineEdit(self.uiGeneralSettingsTabWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiTraceNGPathLineEdit.sizePolicy().hasHeightForWidth())
        self.uiTraceNGPathLineEdit.setSizePolicy(sizePolicy)
        self.uiTraceNGPathLineEdit.setObjectName("uiTraceNGPathLineEdit")
        self.horizontalLayout_5.addWidget(self.uiTraceNGPathLineEdit)
        self.uiTraceNGPathToolButton = QtWidgets.QToolButton(self.uiGeneralSettingsTabWidget)
        self.uiTraceNGPathToolButton.setToolButtonStyle(QtCore.Qt.ToolButtonTextOnly)
        self.uiTraceNGPathToolButton.setObjectName("uiTraceNGPathToolButton")
        self.horizontalLayout_5.addWidget(self.uiTraceNGPathToolButton)
        self.verticalLayout.addLayout(self.horizontalLayout_5)
        spacerItem = QtWidgets.QSpacerItem(20, 5, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout.addItem(spacerItem)
        self.uiTabWidget.addTab(self.uiGeneralSettingsTabWidget, "")
        self.verticalLayout_2.addWidget(self.uiTabWidget)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        spacerItem1 = QtWidgets.QSpacerItem(138, 17, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem1)
        self.uiRestoreDefaultsPushButton = QtWidgets.QPushButton(TraceNGPreferencesPageWidget)
        self.uiRestoreDefaultsPushButton.setObjectName("uiRestoreDefaultsPushButton")
        self.horizontalLayout_2.addWidget(self.uiRestoreDefaultsPushButton)
        self.verticalLayout_2.addLayout(self.horizontalLayout_2)

        self.retranslateUi(TraceNGPreferencesPageWidget)
        self.uiTabWidget.setCurrentIndex(0)
        QtCore.QMetaObject.connectSlotsByName(TraceNGPreferencesPageWidget)

    def retranslateUi(self, TraceNGPreferencesPageWidget):
        _translate = QtCore.QCoreApplication.translate
        TraceNGPreferencesPageWidget.setWindowTitle(_translate("TraceNGPreferencesPageWidget", "TraceNG"))
        self.uiTraceNGPathLabel.setText(_translate("TraceNGPreferencesPageWidget", "Path to TraceNG executable:"))
        self.uiTraceNGPathToolButton.setText(_translate("TraceNGPreferencesPageWidget", "&Browse..."))
        self.uiTabWidget.setTabText(self.uiTabWidget.indexOf(self.uiGeneralSettingsTabWidget), _translate("TraceNGPreferencesPageWidget", "Local settings"))
        self.uiRestoreDefaultsPushButton.setText(_translate("TraceNGPreferencesPageWidget", "Restore defaults"))




# -*- coding: utf-8 -*-
#
# Copyright (C) 2016 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
Default Built-in settings.
"""

from gns3.node import Node

BUILTIN_SETTINGS = {
}


CLOUD_SETTINGS = {
    "name": "",
    "default_name_format": "Cloud{0}",
    "symbol": ":/symbols/cloud.svg",
    "category": Node.end_devices,
    "ports_mapping": [],
}

ETHERNET_HUB_SETTINGS = {
    "name": "",
    "default_name_format": "Hub{0}",
    "symbol": ":/symbols/hub.svg",
    "category": Node.switches,
    "ports_mapping": [],
}

ETHERNET_SWITCH_SETTINGS = {
    "name": "",
    "default_name_format": "Switch{0}",
    "symbol": ":/symbols/ethernet_switch.svg",
    "category": Node.switches,
    "ports_mapping": [],
}



# -*- coding: utf-8 -*-
#
# Copyright (C) 2014 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from ..qt import QtGui, QtWidgets, qslot
from ..ui.filter_dialog_ui import Ui_FilterDialog


class FilterDialog(QtWidgets.QDialog, Ui_FilterDialog):

    """
    Filter dialog.
    """

    def __init__(self, parent, link):

        super().__init__(parent)
        self.setupUi(self)
        self._link = link
        self._link.updated_link_signal.connect(self._updateUiSlot)
        self._link.listAvailableFilters(self._listAvailableFiltersCallback)
        self._initialized = False
        self._filter_items = {}
        self.uiButtonBox.button(QtWidgets.QDialogButtonBox.Apply).clicked.connect(self._applyPreferencesSlot)
        self.uiButtonBox.button(QtWidgets.QDialogButtonBox.Help).clicked.connect(self._helpSlot)
        self.uiButtonBox.button(QtWidgets.QDialogButtonBox.Reset).clicked.connect(self._resetSlot)

    def _listAvailableFiltersCallback(self, result, error=False, *args, **kwargs):
        if error:
            QtWidgets.QMessageBox.warning(None, "Link", "Error while listing information about the link: {}".format(result["message"]))
            return
        self._filters = result
        self._initialized = True
        self._updateUiSlot()

    @qslot
    def _updateUiSlot(self, *args):

        # Empty the main layout
        while True:
            item = self.uiVerticalLayout.takeAt(0)
            if item is None:
                break
            elif item.widget():
                item.widget().deleteLater()

        if len(self._filters) == 0:
            QtWidgets.QMessageBox.critical(self, "Link", "No filter available for this link. Try with a different node type.")
            self.reject()

        self._tabWidget = QtWidgets.QTabWidget(self)
        for i, filter in enumerate(self._filters):
            tab = QtWidgets.QWidget()
            self._tabWidget.addTab(tab, filter['name'])
            self._tabWidget.setTabToolTip(i, filter['description'])
            self._tabWidget.setTabIcon(i, QtGui.QIcon(':/icons/led_red.svg'))
            vlayout = QtWidgets.QVBoxLayout()

            gridLayout = QtWidgets.QGridLayout()
            line = 0
            filter["spinBoxes"] = []
            filter["textEdits"] = []

            nb_spin = 0

            for param in filter["parameters"]:
                label = QtWidgets.QLabel()
                label.setText(param["name"] + ":")
                gridLayout.addWidget(label, line, 0, 1, 1)

                if param["type"] == "int":
                    spinBox = QtWidgets.QSpinBox()
                    filter["spinBoxes"].append(spinBox)
                    spinBox.setMinimum(param["minimum"])
                    spinBox.setMaximum(param["maximum"])

                    sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
                    sizePolicy.setHorizontalStretch(0)
                    sizePolicy.setVerticalStretch(0)
                    sizePolicy.setHeightForWidth(spinBox.sizePolicy().hasHeightForWidth())
                    spinBox.setSizePolicy(sizePolicy)
                    try:
                        value = self._link.filters()[filter["type"]][nb_spin]
                        spinBox.setValue(value)
                        if value != 0:
                            self._tabWidget.setTabIcon(i, QtGui.QIcon(':/icons/led_green.svg'))
                    except(KeyError, IndexError):
                        pass
                    nb_spin += 1
                    gridLayout.addWidget(spinBox, line, 1, 1, 1)
                    unit = QtWidgets.QLabel()
                    unit.setText(param["unit"])
                    gridLayout.addWidget(unit, line, 2, 1, 1)
                elif param["type"] == "text":
                    textEdit = QtWidgets.QTextEdit()
                    textEdit.setAcceptRichText(False)
                    filter["textEdits"].append(textEdit)
                    sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
                    sizePolicy.setHorizontalStretch(0)
                    sizePolicy.setVerticalStretch(0)
                    textEdit.setMinimumWidth(300)
                    textEdit.setSizePolicy(sizePolicy)
                    try:
                        text = self._link.filters()[filter["type"]][0]
                        textEdit.setPlainText(text)
                        if text:
                            self._tabWidget.setTabIcon(i, QtGui.QIcon(':/icons/led_green.svg'))
                    except(KeyError, IndexError):
                        pass
                    gridLayout.addWidget(textEdit, line, 1, 1, 1)

                line += 1

            spacerItem = QtWidgets.QSpacerItem(1, 1, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
            gridLayout.addItem(spacerItem, line, 0, 1, 1)
            vlayout.addLayout(gridLayout)
            tab.setLayout(vlayout)

        self.uiVerticalLayout.addWidget(self._tabWidget)

    @qslot
    def _applyPreferencesSlot(self, *args):
        new_filters = {}
        for filter in self._filters:
            new_filters[filter["type"]] = []
            for spinBox in filter["spinBoxes"]:
                new_filters[filter["type"]].append(spinBox.value())
            for spinBox in filter["textEdits"]:
                new_filters[filter["type"]].append(spinBox.toPlainText())
        self._link.setFilters(new_filters)
        self._link.update()

    @qslot
    def _helpSlot(self, *args):
        help_text = "Filters are applied to packets in both direction.\n\n"

        filter_nb = 0
        for filter in self._filters:
            help_text += "{}: {}".format(filter["name"], filter["description"])
            filter_nb += 1
            if len(self._filters) != filter_nb:
                help_text += "\n\n"

        QtWidgets.QMessageBox.information(self, "Help for filters", help_text)

    @qslot
    def _resetSlot(self, *args):

        filters = {}
        self._link.setFilters(filters)
        self._link.update()

    def done(self, result):
        """
        Called when the dialog is closed.

        :param result: boolean (accepted or rejected)
        """
        if result and self._initialized:
            self._applyPreferencesSlot()
        super().done(result)



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '/home/grossmj/PycharmProjects/gns3-gui/gns3/modules/builtin/ui/cloud_wizard.ui'
#
# Created: Fri Jun 10 15:32:51 2016
#      by: PyQt5 UI code generator 5.2.1
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_CloudNodeWizard(object):
    def setupUi(self, CloudNodeWizard):
        CloudNodeWizard.setObjectName("CloudNodeWizard")
        CloudNodeWizard.resize(585, 423)
        CloudNodeWizard.setModal(True)
        self.uiServerWizardPage = QtWidgets.QWizardPage()
        self.uiServerWizardPage.setObjectName("uiServerWizardPage")
        self.gridLayout_2 = QtWidgets.QGridLayout(self.uiServerWizardPage)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.uiServerTypeGroupBox = QtWidgets.QGroupBox(self.uiServerWizardPage)
        self.uiServerTypeGroupBox.setObjectName("uiServerTypeGroupBox")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.uiServerTypeGroupBox)
        self.verticalLayout.setObjectName("verticalLayout")
        self.uiRemoteRadioButton = QtWidgets.QRadioButton(self.uiServerTypeGroupBox)
        self.uiRemoteRadioButton.setChecked(True)
        self.uiRemoteRadioButton.setObjectName("uiRemoteRadioButton")
        self.verticalLayout.addWidget(self.uiRemoteRadioButton)
        self.uiVMRadioButton = QtWidgets.QRadioButton(self.uiServerTypeGroupBox)
        self.uiVMRadioButton.setObjectName("uiVMRadioButton")
        self.verticalLayout.addWidget(self.uiVMRadioButton)
        self.uiLocalRadioButton = QtWidgets.QRadioButton(self.uiServerTypeGroupBox)
        self.uiLocalRadioButton.setObjectName("uiLocalRadioButton")
        self.verticalLayout.addWidget(self.uiLocalRadioButton)
        self.gridLayout_2.addWidget(self.uiServerTypeGroupBox, 0, 0, 1, 1)
        self.uiRemoteServersGroupBox = QtWidgets.QGroupBox(self.uiServerWizardPage)
        self.uiRemoteServersGroupBox.setObjectName("uiRemoteServersGroupBox")
        self.gridLayout_7 = QtWidgets.QGridLayout(self.uiRemoteServersGroupBox)
        self.gridLayout_7.setObjectName("gridLayout_7")
        self.uiRemoteServersLabel = QtWidgets.QLabel(self.uiRemoteServersGroupBox)
        self.uiRemoteServersLabel.setObjectName("uiRemoteServersLabel")
        self.gridLayout_7.addWidget(self.uiRemoteServersLabel, 0, 0, 1, 1)
        self.uiRemoteServersComboBox = QtWidgets.QComboBox(self.uiRemoteServersGroupBox)
        self.uiRemoteServersComboBox.setEnabled(False)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiRemoteServersComboBox.sizePolicy().hasHeightForWidth())
        self.uiRemoteServersComboBox.setSizePolicy(sizePolicy)
        self.uiRemoteServersComboBox.setObjectName("uiRemoteServersComboBox")
        self.gridLayout_7.addWidget(self.uiRemoteServersComboBox, 0, 1, 1, 1)
        self.gridLayout_2.addWidget(self.uiRemoteServersGroupBox, 1, 0, 1, 1)
        CloudNodeWizard.addPage(self.uiServerWizardPage)
        self.uiNameWizardPage = QtWidgets.QWizardPage()
        self.uiNameWizardPage.setObjectName("uiNameWizardPage")
        self.gridLayout = QtWidgets.QGridLayout(self.uiNameWizardPage)
        self.gridLayout.setObjectName("gridLayout")
        self.uiNameLabel = QtWidgets.QLabel(self.uiNameWizardPage)
        self.uiNameLabel.setObjectName("uiNameLabel")
        self.gridLayout.addWidget(self.uiNameLabel, 0, 0, 1, 1)
        self.uiNameLineEdit = QtWidgets.QLineEdit(self.uiNameWizardPage)
        self.uiNameLineEdit.setObjectName("uiNameLineEdit")
        self.gridLayout.addWidget(self.uiNameLineEdit, 0, 1, 1, 1)
        CloudNodeWizard.addPage(self.uiNameWizardPage)

        self.retranslateUi(CloudNodeWizard)
        QtCore.QMetaObject.connectSlotsByName(CloudNodeWizard)

    def retranslateUi(self, CloudNodeWizard):
        _translate = QtCore.QCoreApplication.translate
        CloudNodeWizard.setWindowTitle(_translate("CloudNodeWizard", "New cloud node template"))
        self.uiServerWizardPage.setTitle(_translate("CloudNodeWizard", "Server"))
        self.uiServerWizardPage.setSubTitle(_translate("CloudNodeWizard", "Please choose a server type to run your new cloud node."))
        self.uiServerTypeGroupBox.setTitle(_translate("CloudNodeWizard", "Server type"))
        self.uiRemoteRadioButton.setText(_translate("CloudNodeWizard", "Run the cloud node on a remote computers"))
        self.uiVMRadioButton.setText(_translate("CloudNodeWizard", "Run the cloud node on the GNS3 VM"))
        self.uiLocalRadioButton.setText(_translate("CloudNodeWizard", "Run the cloud node on your local computer"))
        self.uiRemoteServersGroupBox.setTitle(_translate("CloudNodeWizard", "Remote server"))
        self.uiRemoteServersLabel.setText(_translate("CloudNodeWizard", "Run on:"))
        self.uiNameWizardPage.setTitle(_translate("CloudNodeWizard", "Name"))
        self.uiNameWizardPage.setSubTitle(_translate("CloudNodeWizard", "Please choose a descriptive name for the new cloud node."))
        self.uiNameLabel.setText(_translate("CloudNodeWizard", "Name:"))

